#!/usr/bin/env python3
# -*- coding: utf-8 -*-
#
#	workerImport.py
#	
#	Copyright 2017 krasytod <krasytod@gmail.com>
###


import psycopg2
import psycopg2.extras
from http.server import BaseHTTPRequestHandler, HTTPServer 
from socketserver import ThreadingMixIn
import threading
import argparse,time
import re,signal
import cgi,json,configparser
import workerImport 
import logging ,sys ,os
from logging.handlers import TimedRotatingFileHandler
from logging.handlers import RotatingFileHandler 
from multiprocessing import Process, Lock
from lockfile.pidlockfile import PIDLockFile
from lockfile import AlreadyLocked
import subprocess, signal
import multiprocessing
 
config = ''
logger = logging.getLogger('import_server') 
gl_port= "6060" 
gl_server = "0.0.0.0" 
pidfile = None
pg_cur=""
VERSION = {'ver':"V1.0.2",'date':'2017-06-29'}  # this is daemon version




class GracefulKiller:
	def __init__(self):
		signal.signal(signal.SIGINT, self.exit_gracefully)
		signal.signal(signal.SIGTERM, self.exit_gracefully)

	def exit_gracefully(self,signum, frame):
		try:
			pidfile.break_lock()
			logger.debug("Breaking Pid ")
			print("Breaking Pid ")
			exit(0)
		
		except Exception as e: 
			logger.error("Error in Breaking Pid %s"%e)
			print("Error in Breaking Pid %s"%e)
			exit(1)
		

def setup():
	global config ,logger,gl_port,gl_server,pg_cur
	config = configparser.ConfigParser()
	config.read('config.ini')

	if int(config["log"].get('by_time',0) )== 1:
		handler = TimedRotatingFileHandler(config["log"].get ("import_script_path","import_script.log"),when=config["log"].get ("import_script_when",'m'),
		interval= int( config["log"].get ("import_script_interval",0) ),backupCount= int( config["log"].get ("import_script_backupCount",5)  ) )
	else: # by size
		handler = RotatingFileHandler(config["log"].get ("import_script_path","import_script.log"),maxBytes=int( config["log"].get ("import_script_maxBytes", 1024*1024*128 ) ),
		backupCount= int( config["log"].get ("import_script_backupCount",5)  ) )
	
	formatstr = '%(asctime)s - %(name)s - %(levelname)s - %(message)s'
	formatstr = '%(asctime)s: %(levelname)s:  %(message)s'
	formatter = logging.Formatter(formatstr)
	handler.setFormatter(formatter)
	logger.addHandler(handler)
	gl_port = config["server"].get ("port",'6060')
	gl_server = config["server"].get ("server",'6060')
	
	if int (config["log"].get ("debug_level",0) ) == 1: 
		logger.setLevel(logging.DEBUG)
	else:
		logger.setLevel(logging.INFO)
		
	try:
		pg_conn, pg_cur = connect_to_postgresql(config["POSTGRES"]["hostaddr"],int(config["POSTGRES"]["port"]), config["POSTGRES"]["dbname"], config["POSTGRES"]["user"],config["POSTGRES"]["password"]) #config["POSTGRES"]["password"]
	except Exception as e: 
		print (" Exception in connect_to_postgresql:  %s" % (str(e) ) )
		logger.error(" Exception in connect_to_postgresql:  %s" % (str(e) ) )
		return -1
		

 
class HTTPRequestHandler(BaseHTTPRequestHandler):
	def do_POST(self):
		logger.debug("New POST")
		if None != re.search('/api/v1/importfile/*', self.path):
			#logger.debug("New POST2")
			ctype, pdict = cgi.parse_header(str( self.headers.get('Content-Type')  ) )
			if ctype == 'application/json':
				length = int(self.headers.get('content-length'))
				data = cgi.parse_qs(self.rfile.read(length), keep_blank_values=1)
				dict_bin_str = list( data.keys()) [0].decode('utf-8')
				try:
					dict_bin = json.loads(dict_bin_str)
				except Exception as e: 
					print ( str(e) )
					logger.error( str(e) )
					logger.debug(str(e) )
					self.send_response(400, str(e))
					self.send_header('Content-Type', 'text/html')
					self.end_headers()
					return
				print ("JSON REQUEST: ", str(dict_bin) )
				logger.debug("JSON REQUEST: %s"% str(dict_bin))
				file_path = dict_bin.get('file',None)  #,dict_bin['id'])
				if file_path== None:
					self.send_response(400, 'Bad Request: File key not exist')
					logger.error( 'Bad Request: File key not exist' )
					self.send_header('Content-Type', 'text/html')
					self.end_headers()
					return
				_id  = dict_bin.get('id',None) 
				if _id == None:
					pass
					#logger.debug(' id key is not given')
					#self.send_response(400, 'Bad Request: id key not exist')
					#self.send_header('Content-Type', 'application/json')
					#self.end_headers()
					#return
				logger.debug("File Path: "+file_path)  # + " and ID "+str(_id))
				number , msg = workerImport.doWork(file_path,_id,logger)
				#p = multiprocessing.Process(target=workerImport.doWork, args=(file_path,_id,logger) )
				#p.daemon = True
				#p.start()
				#p.join()
				msg_dict= {'return_number':0,'status':"OK"}
				return_msg = json.dumps(msg_dict)
				print (return_msg)
				logger.debug( 'Returned Post '+ str(return_msg) )
				self.send_response(200, return_msg)
				self.send_header('Content-Type', 'application/json')
				self.end_headers()
				
				
			else:
				data = {}
				print ("No DATA!. self.headers is:",self.headers)
				logger.debug("No DATA!. self.headers is:",self.headers)
				self.send_response(400, 'Bad Request:No DATA!')
				self.send_header('Content-Type', 'text/html')
				self.end_headers()
				return
			#self.send_response(200)
			#self.end_headers()
		else:
			self.send_response(403,"Incorrect URL!!!")
			self.send_header('Content-Type', 'text/html')
			self.end_headers()
			print ("Incorrect URL!!!",self.path)
			logger.debug("Incorrect URL!!!")
		return
 
	def do_GET(self):
		if None != re.search('/api/v1/apitest/*', self.path):
			logging.info ("Get")
			self.send_response(200,"Running")
			
			self.send_header('Content-Type', 'text/html')
			self.end_headers()
			self.wfile.write("running".encode("utf-8"))
		else:
			logging.info ("Get")
			self.send_response(403,"Please do not use GET!")
			self.send_header('Content-Type', 'text/html')
			self.end_headers()
		return
 
class ThreadedHTTPServer(ThreadingMixIn, HTTPServer):
	allow_reuse_address = True
 
	def shutdown(self):
		self.socket.close()
		HTTPServer.shutdown(self)
 
class SimpleHttpServer():
	def __init__(self, ip, port):
		self.server = ThreadedHTTPServer((ip,port), HTTPRequestHandler)
	
	def start(self):
		self.server_thread = threading.Thread(target=self.server.serve_forever)
		self.server_thread.daemon = True
		self.server_thread.start()
 
	def waitForThread(self):
		self.server_thread.join()
 
	def stop(self):
		self.server.shutdown()
		self.waitForThread()
 
def checkPid():
	global pidfile
	pidf = "pid_import.pid"
	pidfile = PIDLockFile(pidf, timeout=-1)
	try:
		#script = subprocess.run("./cf.py", shell=True)
		pidfile.acquire()
		print("Lock acquired!")
	except AlreadyLocked:
		try:
			os.kill(pidfile.read_pid(), signal.SIGTERM)
			print ('Process already running! Trying to kill process.')
			logger.error('Process already running!')
			time.sleep(3)
			try:
				pidfile.acquire()
				print("Lock aquered!")
			except Exception as e:  #No process with locked PID
				print ("OS Error. Please restart service ,check and delete any pid files in current path" )
				logger.error("OS Error. Please restart service ,check and delete any pid files in current path")
				pidfile.break_lock()
				exit(1)
			
		except OSError:  #No process with locked PID
			print ("OS Error.No process with locked PID. Please restart service ,check and delete any pid files in current path" )
			logger.error("OS Error. No process with locked PID.Please restart service ,check and delete any pid files in current path")
			exit(1)
			pidfile.break_lock()
			
			#logger.error('No process with locked PID')
	
 
def run (daemon = True):
	
	logger.info("Starting daemon ...")
	print("Starting daemon ...")
	file_name = os.path.basename(__file__)
	if daemon:
		cmd= "nohup python3 %s --action nur </dev/null >/dev/null 2>&1 &" % file_name
	else:
		cmd= "python3 %s --action nur" % file_name
	# nohup command </dev/null >/dev/null 2>&1 &   nohup python3 alert_rules.py
	proc = subprocess.Popen([cmd], shell = True,stdin=None, stdout=None,stderr=None, close_fds=True)
	logger.info("Import Daemon is started")
	
def stop():
	
	pidfile_import = PIDLockFile("pid_import.pid", timeout=-1)
	print (pidfile_import.read_pid())
	try:
		os.kill(pidfile_import.read_pid(), signal.SIGTERM)
		print ('Process Killed!')
		exit(1)

	except Exception as e:  #No process with locked PID
		logger.error( "No process with locked PID. Probably you must start the daemon first %s" %e )
		print ("No process with locked PID. Probably you must start the daemon first %s" %e )

def connect_to_postgresql(host, port, database, user, password=None):
	conn = psycopg2.connect(host=host, port=port, database=database, user=user, password=password)
	conn.autocommit = True
	cursor = conn.cursor(cursor_factory=psycopg2.extras.DictCursor)
	print("connected to DB" )
		
	return conn, cursor


def git_version():
    from subprocess import Popen, PIPE
    gitproc = Popen(['git', 'rev-parse','HEAD'], stdout = PIPE)
    (stdout, _) = gitproc.communicate()
    return stdout.strip()


def logVersion():
	global pg_cur
	worker_info = workerImport.getVersion()  #VERSION = ['ver':"1.0.1",'date':'2017-05-25']
	now_time = time.strftime("%Y-%m-%d %H:%M:%S", time.gmtime())+"+00"
	program_name_dmon = "dnl_import"
	program_name_proc = "dnl_import"
	git_ver = "" # str( git_version()  , 'utf-8')
	
	#requires POSTGRESS ver > 9.5
	
	sql = """   SELECT count(*) as count FROM version_information WHERE program_name = '%s'""" % program_name_dmon
	print(sql)
	pg_cur.execute(sql)
	count = pg_cur.fetchone()['count']
	if count == 0:
		sql = """INSERT INTO version_information (program_name ,major_ver,build_date ,start_time ) VALUES (%s,%s,%s,%s)  """
	
	else:
		sql = """UPDATE  version_information  SET  major_ver = '%s' , build_date = '%s',start_time = '%s' WHERE  program_name = '%s'; """ % (VERSION['ver'],VERSION['date'],now_time,program_name_dmon) 
	
	print (sql)
	try:
		pg_cur.execute(sql, (program_name_dmon, VERSION['ver'],VERSION['date'],now_time)  )
		
	
	except Exception as e:  
		print ("Except in versioning %s"%e )
		logger.error("Except in versioning %s"%e )
		

	
if __name__=='__main__':
	if os.geteuid() == 0:
		exit("You do not need to have root privileges to run this script.\nPlease try again, this time without using sudo. Exiting.")
	killer = GracefulKiller()
	parser = argparse.ArgumentParser(description='Import API')
	parser.add_argument('-a', '--action',  help='Action to be taken. start , stop')
	args = parser.parse_args()
	setup()
	logger.info (str(args))
	if args.action == "start":
		run ()
		exit(0)
	elif args.action == "stop":
		stop()
		exit(0)

	elif args.action == "debug":
		print("starting in foreground")
		#run (False)
		#exit(0)

	elif args.action == "nur":
		print("starting daemon")
		
	else:
		print ("Unrecognized command")
		exit(0)
	
	
	#args = parser.parse_args()
	checkPid() 
	logVersion()
	server = SimpleHttpServer(gl_server, int(gl_port))
	print ('HTTP Server Running on %s:%s'  % (gl_server, int(gl_port)) )
	
	logger.info ('HTTP Server Running on %s:%s'  % (gl_server, int(gl_port)) )
	
	server.start()
	server.waitForThread()
