#!/usr/bin/env python3
# -*- coding: utf-8 -*-
#
#  workerImport.py
#  25-05-2017
#  Copyright 2017 krasytod <krasytod@gmail.com>
###

VERSION = {'ver':"V1.4.1a",'date':'2018-09-09'}


import psycopg2
import psycopg2.extras
import datetime,time,re
import configparser , csv,xlrd , pytz
import logging ,os #,sys
from logging.handlers import TimedRotatingFileHandler
from logging.handlers import RotatingFileHandler
from lockfile.pidlockfile import PIDLockFile
from lockfile import AlreadyLocked
from copy import deepcopy
import traceback
from phpserialize import *
import codecs
pg_cur = ""
config = ''
from io import StringIO

MATCH_CODE_DECK_NAME = r"\A\d+-\d+\Z"
MATCH_CODE_DECK_COUNTRY =r"\A[A-Z]+\Z"
MATCH_ONLY_NUMBERS = r'\A\d*$'
MATCH_ONLY_NUMBERS_AND_ALPHABET = r'\A\w*$'
MATCH_ONLY_ALPHABET = r'\d'
MATCH_CODE_DECK_NAME_OTHER  =  r'\A[\w\s\u0028\u0029\u002D\u002F\u005C\x00\x28\x29\x2D\x2F\x96\x97\xAD]*$'              #r'\A[\w\s]*$'

logger = logging.getLogger('import_script')


#python3 current_dir/class4_ocn_lata.py -c  $opts{c} -i $row->{rates_file} -o $row->{rates_file_cmd} -d $row->{date_format} -r $rate_table_id";

def getVersion():
	return VERSION


def connect_to_postgresql(host, port, database, user, password=None):
	conn = psycopg2.connect(host=host, port=port, database=database, user=user, password=password)
	conn.autocommit = True
	cursor = conn.cursor(cursor_factory=psycopg2.extras.DictCursor)
	print("connected to DB" )
		
	return conn, cursor


def stripSpace(row):
	"""strips whitespace in the begining and the end of all elements in the row"""
	new_row = []
	for elem in row:
		new_elem =  elem.strip()
		new_row.append(new_elem)
	return new_row 

def DoError(row,message,status=-2):
	
	if status == -1:  # -1 is not recognized from UI
		status= -4
	
	error_file_path = row.get('error_file_path',None)
	print('Writing in Error file !! msg: %s  status: %s' %  (message,status) )
	try:
		f = open(error_file_path, 'a')
	except IOError:
		UpdateStatus(status,row['id']) # status = -2 , missing file !
		print('error in DoError')
	else:
		#UpdateStatus(,row['id'] )
		now_str = '{0:%Y-%m-%d %H:%M:%S: }'.format(datetime.datetime.now())
		
		with f as csvfile:
			csvfile.write("\n")
			csvfile.write(now_str + message)
			csvfile.flush()
				

def CheckCsvError(row,header_len,scv_list):
	#('86','china','china ','') ,('121','Bul','bulgaria','error hope') ,('','trans','','') 
	scv = scv_list.split(',(')  #replace ('(',"").replace (')',"").strip()
	new_list = []
	counter = 1
	logger.debug("CheckCsvError")
	
	for _row in scv:
		_row = _row.replace ('(',"").replace (')',"").strip()
		_row = _row.split(',')
		logger.debug(str(_row[-1]))
		if _row[-1] != "''" :
			logger.debug("Please check line %s for errors" % counter)
			DoError(row, "Please check line %s for errors" % counter )
		counter = counter+1
	logger.debug(str(scv) + str(type(scv) ))




def CheckHeader(scv_list,row,csv_reader_list):
	print ("Checking header:")
	scv = scv_list[0].replace ('(',"").replace (')',"").strip()
	scv_header_list = scv.split(',')
	print ("scv_header_list",scv_header_list)
	is_it_empty_header =True
	for elem in  scv_header_list:
		if len(elem.strip() ) >0:
				is_it_empty_header =False
				break
					#print ("is_it_empty_header: ",str (is_it_empty_header) )
	#logger.debug("scv_header_list:" + str( scv_header_list) + str(len(scv_header_list)) + str(is_it_empty_header ) )
	
	
	
	if is_it_empty_header:
		print ("is_it_empty_header") 
		CheckCsvError(row,len(scv_header_list),scv_list[1])
		return False,csv_reader_list
	table_headers= {
	'Code' : ['code','name','country',""], #['code','country','name']
	'JurisdictionUpload':['jurisdiction_country_name','jurisdiction_name','prefix','ocn','lata',""],
	'RandomAniGeneration':['ani',""],
	 # ANI Min 7 , Ani max 8 ,9 dnis min, 10 dnis max, IC 0 , IT 1 , EC 2 , ET 3 , 5 digit, time 6 ,ani_prefix 4
	'ResourceBlock': ['ingress carrier','ingress trunk' ,'egress carrier', 'egress trunk' ,'ani','dnis','time profile', 'ani length min',  'ani length max','dnis length min',
	'dnis length max', 'block no ani',"blocked by","updated by", "type" ,"create time",""] ,
	'Route': [ 'ani_prefix','ani_min_length','ani_max_length','dnis_prefix','dnis_min_length','dnis_max_length','route_type','dynamic_route_name','static_route_name',
	'intra_static_route_name','inter_static_route_name','jurisdiction_country',""],
	'Productitem': ['code','strategy','trunk_name','percentage','time_profile_name',""],
	'Egress':['trunk_name','carrier_name','media_type','call_limit','cps_limit','pdd_timeout','active','rate_table_name','host_route_strategy','rfc2833','pass_dip_head','min_duration','max_duration',""],
	'Ingress': ['trunk_name','carrier_name','media_type','cps_limit','call_limit','pdd_timeout','ignore_early_media','active','rfc2833','dip_from','min_duration','max_duration',""],
	'EgressPort':['trunk_name','ip','port',""],
	'IngressPort':['trunk_name','ip','port',""],
	'ResourceIp': [ 'trunk_name','ip','port',""] , 'DigitTranslation': [ 'ani_method','ani','translated_ani','dnis_method','dnis','translated_dnis',""] ,
	'DID': ['did','vendor','client','vendor_billing_plan','client_billing_plan',""] , # 
	'DID_SHORT': ['did','vendor_billing_plan','vendor',""],
	'ResourceDirection': ["trunk_name","time_profile_name","target","code","action","chars","number_type","number_length",""],
	'ResourceTranslation': ["trunk_name","translation_name","time_profile_name",''] 
	}
	'''if len(table_headers [ row['obj'] ]) -1!= len( scv_header_list ):
		print ("The size of headers do not match ", len(table_headers [ row['obj'] ]) , len( scv_header_list ) )
		print (table_headers [ row['obj'] ], row['obj'])
		print (scv_header_list )
		return False'''
	cnt = 0
	exchange_list = []
	if row['upload_type'] == 21: # no headers
		return True ,csv_reader_list
	for header in scv_header_list :
		if header.lower().strip() not in table_headers [ row['obj'] ]:
			print (header,len(header), " Not in ", table_headers [ row['obj'] ])
			DoError(row,"Header  %s not in %s !"% ( header  ,str(table_headers [ row['obj'] ]) ) )
			return False,csv_reader_list
		cnt += 1
		if header == "":
			print("MUST NULL!!")
	csv_reader_list = NoneTheColumns(scv_header_list,csv_reader_list,table_headers [ row['obj'] ])
	if csv_reader_list == 1:
		return False,csv_reader_list
	#print (csv_reader_list)
	return True ,csv_reader_list

def IgnoreOrDeleteDoImportJurPrefix (scv_list,row,id_data,table_name,table_id_data,csv_reader_list) :
	print ("Entering IgnoreOrDeleteDoImportJurPrefix")
	status = 0 
	msg = "Done"
	
	table_name = {
	'Code' : 'code',
	'JurisdictionUpload':'jurisdiction_prefix',
	'ResourceBlock':'ResourceBlock'
	}
	table_return = {
	'Code' : 'code_id',
	'JurisdictionUpload':'id'
		}
	table_id_data = {
	'Code' : 'code_deck_id',
	'JurisdictionUpload':'prefix'
		}
	
	action =  row.get('duplicate_type',None)
	#print ("action type is %s" % action)
	#print (scv_list , csv_reader_list)
	headers_string = scv_list[0].strip()  #'(jurisdiction_country_name,jurisdiction_name,prefix,,lata) '
	headers_string = headers_string[1:-1 ]
	headers = headers_string. split(',')
	print ("header length",len(headers))
	some_error= False
	num_succ  =0
	num_fail  =0
	num_dubb  =0
	
	# inserting new record 
	#anne [4:13 PM] i think jurisdiciton country [4:13] if it is empty [4:14] you need to insert 1 there
	sql = " INSERT INTO  jurisdiction_country (name,id) VALUES ('US',1) "
	try:
		pg_cur.execute(sql)
	except Exception as e:
		print("Already exists",e)
	
	if action == None:
		action = 'delete'
	try:
		
		if action == 'ignore':  # loop over every row in csv to decide if the row to be included in insert or not
			counter= 1
			
			for csv_row in csv_reader_list:  # loop over all rows 
				csv_row =stripSpace(csv_row)
				insert_date  = datetime.datetime.now(pytz.timezone('GMT')).strftime('%Y-%m-%d %H:%M:%S %z')
				if len(csv_row ) == 0:
					msg = " Possible empty rows:  %s  on row: %s" % (str(csv_row), counter    )
					DoError(row,msg,0) 
					#some_error= True
					num_fail = num_fail +1 
					counter = counter + 1
					continue
				if len(csv_row[2] ) == 0:
					msg = " Possible empty rows:  %s  on row: %s" % (str(csv_row), counter    )
					DoError(row,msg,0) 
					#some_error= True
					num_fail = num_fail +1 
					counter = counter + 1
					continue
				
				
				# only numbers for prefix    ///, ocn and lata
				try:
					#int(csv_row[3])
					#int(csv_row[4])
					int(csv_row[2])
				except:  #it is not only numbers
					msg = " Error in a row:  %s  with number row: %s.   prefix must contain only numbers" % (str(csv_row), counter    )
					DoError(row,msg,0) 
					#some_error= True
					num_fail = num_fail +1 
					counter = counter + 1
					continue
				
				
				
				
				prefix =  "'%s'" % csv_row[2]
				if len(csv_row[0] ) > 100 or len(csv_row[1] )> 100 or len(csv_row[3] )> 10 or len(csv_row[4] ) > 10:
					status = -1
					limits = ""
					if len(csv_row[0] )> 100:
						limits = limits +  " Jurisdiction_country_name is bigger from 100 chars."
					if len(csv_row[1] )> 100:
						limits = limits +   " Jurisdiction_name is bigger from 100 chars."
					if len(csv_row[3] ) > 10:
						limits = limits +  " OCN is bigger from 10 chars."
					if len(csv_row[4] )> 100:
						limits = limits +  " LATA is bigger from 10 chars."
					print ( "Data is too big: row %s"  %  counter )
					logger.debug( "Data is too big: row %s"  %  counter )
					message = "Error in line %s. %s " % (counter,limits )
					msg = "Done with errors.Please check log error file"
					DoError(row,message,-1) 
					some_error= True
					num_fail = num_fail +1 
					#UpdateStatus(-1,row['id'])
					continue
				
				#prefix check for special symbols
				matchObj = re.match(MATCH_ONLY_NUMBERS_AND_ALPHABET , csv_row[2].replace(" ", ""), re.M|re.I)
				if not matchObj:
					msg = "Prefix is with wrong format: %s  in row number: %s. Hint. It must contain only numbers and alphahbet symbols" % ( csv_row[2], counter )
					DoError(row,msg,-1) 
					num_fail = num_fail +1 
					counter = counter + 1
					continue
				
				#countr  check for special symbols .  it is allowed to have space
				matchObj = re.match(MATCH_ONLY_NUMBERS_AND_ALPHABET , csv_row[0].replace(" ", ""), re.M|re.I)
				if not matchObj:
					msg = "Country is with wrong format: %s  in row number: %s. Hint. It must contain only alphahbet symbols" % ( csv_row[0], counter )
					DoError(row,msg,-1) 
					num_fail = num_fail +1 
					counter = counter + 1
					continue
				
				matchObj = re.match(MATCH_ONLY_ALPHABET  , csv_row[0].replace(" ", ""), re.M|re.I)
				if  matchObj:
					msg = "Country is with wrong format: %s  in row number: %s. Hint. It must contain only alphahbet symbols" % ( csv_row[0], counter )
					DoError(row,msg,-1) 
					num_fail = num_fail +1 
					counter = counter + 1
					continue
				
				
				#do not use this!!!
				sql_country = """  with s as (    select id, "name"  from jurisdiction_country     where name = '%s' 
				), i as (     insert into jurisdiction_country ("name")    select 'name'  where not exists (select 1 from s)    returning id, "name")
				select id, "name" from i union all select id, "name"  from s """  % csv_row[0]
				
				#print(sql_country)
				#pg_cur.execute(sql_country)
				_id =   1 #pg_cur.fetchone()['id']
				#print (_id)
				
				sql = """insert into jurisdiction (alias,name,jurisdiction_country_id) VALUES (%s,%s,%s) returning id"""
				#print (sql)
				try:
					pg_cur.execute(sql,(csv_row[0],csv_row[0],_id  ))
					jur_id = pg_cur.fetchone()['id']
				except:  # possible unique constrain violation
					sql = """select id from  jurisdiction where alias = '%s' and name ='%s' """ % (csv_row[0],csv_row[0])
					pg_cur.execute(sql)
					jur_id = pg_cur.fetchone()
					if jur_id != None:
						jur_id = pg_cur.fetchone()['id']
					else:
						num_fail = num_fail +1 
						counter = counter + 1
						continue
				
				sql = "select count(*) as count from %s where %s = %s and jurisdiction_country_id = %s and jurisdiction_id = %s" % ('jurisdiction_prefix' , 'prefix', 
				 prefix, _id,jur_id  )
				#print ( "SQL FOR IGNORE JUR PRE option: %s"  %  sql )
				logger.debug( "SQL FOR IGNORE JUR PRE option: %s"  %  sql )
				pg_cur.execute(sql)
				count= pg_cur.fetchone()[0]
				#print ("count: ",count)
				if count >0 :
					message = "Duplicate row:  %s" % str(csv_row)
					DoError(row,message,0) 
					num_dubb = num_dubb +1
					continue
				counter = counter + 1
				
				
				
				sql = """INSERT INTO jurisdiction_prefix ( jurisdiction_name ,  prefix, jurisdiction_country_id,  jurisdiction_country_name ,ocn,lata,jurisdiction_id,effective_date ) 
				VALUES  (%s,%s,%s,%s,%s,%s,%s,%s) """
				#print ( "SQL FOR INSERT IN JUR PREFIX: %s"  %  sql )
				logger.debug( "SQL FOR INSERT IN JUR PREFIX:: %s"  %  sql )
				pg_cur.execute(sql, ( csv_row[1],csv_row[2],_id,csv_row[0],csv_row[3],csv_row[4], jur_id,insert_date  )   ) 
				num_succ = num_succ +1
				if counter% 1000 == 0 :
					print ( counter,num_succ,num_fail,num_dubb )
			if some_error== False:
				UpdateStatus(2,row['id'],num_succ,num_fail,num_dubb)
			else:
				UpdateStatus(-1,row['id'],num_succ,num_fail,num_dubb)
			return [status,msg]
				
		else:
			counter= 1
			print ("length of csv_reader_list",len(csv_reader_list ))
			for csv_row in csv_reader_list:  # loop over all rows
				#print (counter)
				try:
					csv_row =stripSpace(csv_row)
				except Exception as e:
					print (e)
					num_fail = num_fail +1 
					counter = counter + 1
					continue
					
				insert_date  = datetime.datetime.now(pytz.timezone('GMT')).strftime('%Y-%m-%d %H:%M:%S %z')
				if len(csv_row ) == 0:
					msg = " Possible empty rows:  %s  on row: %s" % (str(csv_row), counter    )
					DoError(row,msg,0) 
					#some_error= True
					num_fail = num_fail +1 
					counter = counter + 1
					continue
				if len(csv_row[2] ) == 0:
					msg = " Possible empty rows:  %s  on row: %s" % (str(csv_row), counter    )
					DoError(row,msg,0) 
					#some_error= True
					num_fail = num_fail +1 
					counter = counter + 1
					continue
				
				
				
				# only numbers for prefix , ocn and lata
				try:
					#int(csv_row[3])
					#int(csv_row[4])
					int(csv_row[2])
				except:  #it is not only numbers
					msg = " Error in a row:  %s  with number row: %s.   prefix must contain only numbers" % (str(csv_row), counter    )
					DoError(row,msg,0) 
					#some_error= True
					num_fail = num_fail +1 
					counter = counter + 1
					continue
				
				prefix =  "'%s'" % csv_row[2]
				if len(csv_row[0] ) > 100 or len(csv_row[1] )> 100 or len(csv_row[3] )> 10 or len(csv_row[4] ) > 10:
					status = -1
					limits = ""
					if len(csv_row[0] )> 100:
						limits = limits +  " Jurisdiction_country_name is bigger from 100 chars."
					if len(csv_row[1] )> 100:
						limits = limits +   " Jurisdiction_name is bigger from 100 chars."
					if len(csv_row[3] ) > 10:
						limits = limits +  " OCN is bigger from 10 chars."
					if len(csv_row[4] )> 100:
						limits = limits +  " LATA is bigger from 10 chars."
					print ( "Data is too big: row %s"  %  counter )
					logger.debug( "Data is too big: row %s"  %  counter )
					message = "Error in line %s. %s " % (counter,limits )
					msg = "Done with errors.Please check log error file"
					DoError(row,message,-1) 
					some_error= True
					num_fail = num_fail +1 
					#UpdateStatus(-1,row['id'])
					continue
				
				#print ("matching phase")
				matchObj = re.match(MATCH_ONLY_NUMBERS_AND_ALPHABET , csv_row[2].replace(" ", ""), re.M|re.I)
				if not matchObj:
					msg = "Prefix is with wrong format: %s  in row number: %s. Hint. It must contain only numbers and alphahbet symbols" % ( csv_row[2], counter)
					DoError(row,msg,-1) 
					num_fail = num_fail +1 
					counter = counter + 1
					continue
				
				#countr  check for special symbols .  it is allowed to have space
				matchObj = re.match(MATCH_ONLY_NUMBERS_AND_ALPHABET , csv_row[0].replace(" ", ""), re.M|re.I)
				if not matchObj:
					msg = "Prefix is with wrong format: %s  in row number: %s. Hint. It must contain only numbers and alphahbet symbols" % ( csv_row[0], counter )
					DoError(row,msg,-1) 
					num_fail = num_fail +1 
					counter = counter + 1
					continue
				
				matchObj = re.match(MATCH_ONLY_ALPHABET  , csv_row[0].replace(" ", ""), re.M|re.I)
				if  matchObj:
					msg = "Country is with wrong format: %s  in row number: %s. Hint. It must contain only alphahbet symbols" % ( csv_row[0], counter )
					DoError(row,msg,-1) 
					num_fail = num_fail +1 
					counter = counter + 1
					continue
				
				
				sql_country = """  with s as (    select id, "name"  from jurisdiction_country     where name = '%s' 
				), i as (     insert into jurisdiction_country ("name")    select 'name'  where not exists (select 1 from s)    returning id, "name")
				select id, "name" from i union all select id, "name"  from s """  % csv_row[0]
				
				#print(sql_country)
				#pg_cur.execute(sql_country)
				_id = 1 #pg_cur.fetchone()['id']
				#print ("setting jur id to 1")
				
				
				sql = """select id from  jurisdiction where alias = '%s' and name ='%s' """ % (csv_row[0],csv_row[0])
				#print (sql,csv_row[0])
				try:
					pg_cur.execute(sql)
					jur_id = pg_cur.fetchone()['id']

				except Exception as e:  # missing jur
					print ("missing jur " ,e)
					sql = """insert into jurisdiction (alias,name,jurisdiction_country_id) VALUES (%s,%s,%s) returning id"""
					#print (sql)
					try:
						pg_cur.execute(sql,(csv_row[0],csv_row[0],_id  ))
						jur_id = pg_cur.fetchone()
					except Exception as e:  # missing jur
						print ("missing jur 1" ,e)
						num_fail = num_fail +1 
						counter = counter + 1
						continue
					if jur_id != None:
						jur_id = pg_cur.fetchone()['id']
					else:
						num_fail = num_fail +1 
						counter = counter + 1
						continue
				
				sql = "DELETE FROM %s where %s = %s and jurisdiction_country_id = %s and jurisdiction_id = %s" % ('jurisdiction_prefix' , 'prefix', 
				 prefix, _id,jur_id  )
				#print ( "SQL FOR IGNORE JUR PRE option: %s"  %  sql )
				#logger.debug( "SQL FOR IGNORE JUR PRE option: %s"  %  sql )
				pg_cur.execute(sql)
				if pg_cur.rowcount > 0 :
					num_dubb = num_dubb +1
					num_succ = num_succ -1
				else:
					pass
				counter = counter + 1

				sql = """INSERT INTO jurisdiction_prefix ( jurisdiction_name,  prefix, jurisdiction_country_id,  jurisdiction_country_name ,ocn,lata,jurisdiction_id,effective_date ) 
				VALUES  (%s,%s,%s,%s,%s,%s,%s,%s) """
				#print ( "SQL FOR INSERT IN JUR PREFIX: %s"  %  sql )
				#logger.debug( "SQL FOR INSERT IN JUR PREFIX:: %s"  %  sql )
				pg_cur.execute(sql, ( csv_row[1],csv_row[2],_id,csv_row[0],csv_row[3],csv_row[4], jur_id ,insert_date )   ) 
				num_succ = num_succ +1
				if counter% 100 == 0 :
					print ( counter,num_succ,num_fail,num_dubb )
					UpdateStatus(1,row['id'],num_succ,num_fail,num_dubb)
			if some_error== False:
				UpdateStatus(2,row['id'],num_succ,num_fail,num_dubb)
			else:
				UpdateStatus(-1,row['id'],num_succ,num_fail,num_dubb)
			return [status,msg]			
		return [0,"Done"]
	except Exception as e: 
		print (" Exception in IgnoreOrDeleteDoImportJurPrefix:  %s on line: %s" % (str(e),counter ) )
		logger.error(" Exception in IgnoreOrDeleteDoImportJurPrefix:  %s" % (str(e) ) )
		DoError(row," Exception in IgnoreOrDeleteDoImportJurPrefix function:  %s" % (str(e)) )
		UpdateStatus(-5,row['id'])
		return [-2,"Error in Importing"]




def data2csv(data):
    si = StringIO()
    cw = csv.writer(si, delimiter='\t',lineterminator="\n")
    for row in data:
        print ("row", row)
        r = [ (x is None and '\n' or x) for x in row]
        print (r)
        cw.writerow(r)
    si.seek(0)
    return si # .getvalue()

def upsert(pg_cur, table_name, selector_fields, setter_fields, data):

    csv_data = data2csv(data)

    sql_template = """
        WITH updates AS (
            UPDATE %(target)s t
                SET %(set)s        
            FROM importsource s
            WHERE %(where_t_pk_eq_s_pk)s 
            RETURNING %(s_pk)s
        )
        INSERT INTO %(target)s (%(columns)s)
            SELECT %(source_columns)s 
            FROM importsource s LEFT JOIN updates t USING(%(pk)s)
            WHERE %(where_t_pk_is_null)s
            GROUP BY %(s_pk)s
    """
    statement = sql_template % dict(
        target = table_name,
        set = ',\n'.join(["%s = s.%s" % (x,x) for x in setter_fields]),
        where_t_pk_eq_s_pk = ' AND '.join(["t.%s = s.%s" % (x,x) for x in selector_fields]),
        s_pk = ','.join(["s.%s" % x for x in selector_fields]),
        columns = ','.join([x for x in selector_fields]),   #+setter_fields
        source_columns = ','.join(['s.%s' % x for x in selector_fields]),   #+setter_fields
        pk = ','.join(selector_fields),
        where_t_pk_is_null = ' AND '.join(["t.%s IS NULL" % x for x in selector_fields]),
        t_pk = ','.join(["t.%s" % x for x in selector_fields]))

    logging.debug(statement) 
    print (statement)

    # with cursor as cur:
    #cur = cursor
    #print('CREATE TEMP TABLE import_source(LIKE %s INCLUDING ALL) ;' % table_name)  #ON COMMIT DROP
    #cur.execute('CREATE TEMP TABLE import_source(LIKE %s INCLUDING ALL) ON COMMIT DROP;' % table_name);  
    #pg_cur.execute("select * from importsource" )
    #pg_cur.fetchone()
    
    #selector_fields = ['code','code_deck_id','country','name']
    #setter_fields = ['code','code_deck_id','country','name']
    pg_cur.copy_from(csv_data, 'importsource', columns=selector_fields)  #+setter_fields
    pg_cur.execute(statement)
    pg_cur.execute('DROP TABLE importsource')
    csv_data.close()




def IgnoreOrDeleteDoImportCodeDeck (scv_list,row,id_data,table_name,table_id_data,csv_reader_list) :
	print ("Entering IgnoreOrDeleteDoImportCodeDeck")
	status = 0 
	msg = ""
	action =  row.get('duplicate_type',None)
	print ("action type is %s" % action)
	some_error= False
	num_succ  =0
	num_fail  =0
	num_dubb  =0
	code_deck_id = row.get('foreign_id',None)
	if code_deck_id == None:
		UpdateStatus(-5,row['id'])
		print ("foreign_id is missing. Abort!") 
		return [-1,"foreign_id is missing. Abort!"]
	if action == None:
		action = 'delete'
	print ("csv_reader_list len: ",len(csv_reader_list))
	
	code_dict = dict()  # key: code, values [code_deck_id,country,name,,counter]
	counter= 1
	for csv_row in csv_reader_list:  
		csv_row =stripSpace(csv_row)
		if len(csv_row ) == 0:
			msg = "Empty row:  %s  on row: %s" % (str(csv_row), counter    )
			DoError(row,msg,-1) 
			#some_error= True
			num_fail = num_fail +1 
			counter = counter + 1
			continue
		if len(csv_row[0] ) == 0 or len(csv_row[1] )  == 0 :
			status = -1
			msg = " Possible empty rows:  %s  on row: %s" % (str(csv_row), counter    )
			DoError(row,msg,-1) 
			#some_error= True
			num_fail = num_fail +1 
			counter = counter + 1
			continue
		try: 
			int(csv_row[0] )
		except:
			msg = "Code is not a number: %s  in row number: %s" % ( csv_row[0], counter )
			DoError(row,msg,-1) 
			num_fail = num_fail +1 
			counter = counter + 1
			continue
			
		matchObj = re.match(MATCH_CODE_DECK_NAME , csv_row[1].replace(" ", ""), re.M|re.I)
		if not matchObj:
			
			print ("Not matched on first instance")
			matchObj = re.match(MATCH_CODE_DECK_NAME_OTHER , csv_row[1].replace(" ", ""), re.M|re.I)
			if not matchObj:
				msg = "Name is with wrong format: %s in row number:%s" % ( csv_row[1], counter )
				DoError(row,msg,-1) 
				num_fail = num_fail +1 
				counter = counter + 1
				continue
			
			
			
		matchObj = re.match(MATCH_CODE_DECK_COUNTRY , csv_row[2].replace(" ", ""), re.M|re.I)
		if not matchObj:
			msg = "Country is with wrong format: %s  in row number: %s" % ( csv_row[2], counter )
			DoError(row,msg,-1) 
			num_fail = num_fail +1 
			counter = counter + 1
			continue
			
		if len(csv_row[1] ) > 100 or len(csv_row[2] )> 100 :
			status = -1
			limits = ""
			if len(csv_row[2] )> 100:
				limits = limits +  " Code country is bigger from 100 chars."
			if len(csv_row[1] )> 100:
				limits = limits +   " Code name is bigger from 100 chars."
			print ( "Data is too big: row %s"  %  counter )
			logger.debug( "Data is too big: row %s"  %  counter )
			message = "Error in line %s. %s " % (counter,limits )
			msg = "Done with errors.Please check log error file"
			DoError(row,message,-1) 
			some_error= True
			num_fail = num_fail +1 
			counter = counter + 1
			#UpdateStatus(-1,row['id'])
			continue
		code = csv_row[0]
		#code_deck_id 
		country = csv_row[1]
		name = csv_row[2] 
		## key: code, values [code_deck_id,country,name]
		if code in code_dict:
			msg = "Duplicate row:  %s  at row %s" % (str(csv_row), counter)
			DoError(row,msg,0) 
			num_dubb = num_dubb +1
			counter = counter + 1
		else:
			code_dict[code] = [ code_deck_id,country,name,counter]
			counter = counter + 1
	print ("Testing Import source")
	try:
		pg_cur.execute('CREATE TABLE importsource(LIKE code INCLUDING ALL) ;')
	#pg_cur.execute('select * from  importsource')
	except:
		print (" importsource exists")
	try:
		sql =  "select code from code where code_deck_id = %s " % code_deck_id
		logger.debug( "SQL FOR GETTING  CODES  %s"  %  sql )
		pg_cur.execute(sql)
		db_codes= pg_cur.fetchall()
		
	except Exception as e: 
		traceback.print_exc()
		print (" Exception in IgnoreOrDeleteDoImportCodeDeck:  %s" % (str(e) ) )
		logger.error(" Exception in IgnoreOrDeleteDoImportCodeDeck:  %s" % (str(e) ) )
		DoError(row," Exception in IgnoreOrDeleteDoImportCodeDeck function:  %s" % (str(e)) )
		UpdateStatus(-5,row['id'])
		return [-2,"Error in Importing"]
		
	db_code_set = set()  # just a set //Keeps codes from DB
	duplicate_codes_set = set()
	if db_codes != None:
		for code in db_codes:
			db_code_set .add (  code ['code']   )
	
	for code,value in code_dict.items():
		if code in db_code_set:  # this code is already in DB with same code_deck_id
			if action == 'ignore':  # loop over every row in csv to decide if the row to be included in insert or 	
				msg = "Duplicate code:  %s  at row %s" % (code, value[-1] )
				DoError(row,msg,0) 
				num_dubb = num_dubb +1
				code_dict.pop(code, None)
			else:
				msg = "Duplicate code:  %s  at row %s" % (code, value[-1] )
				DoError(row,msg,0) 
				num_dubb = num_dubb +1
				duplicate_codes_set.add (code)
	
	table_name = 'code'
	selector_fields = ['code','code_deck_id','country','name']
	setter_fields = ['code','code_deck_id','country','name']
	data = []
	print ("code_dict",code_dict)
	try:
		for key, value in code_dict.items():
			new_row = list()
			new_row.append(key)
			new_row.extend ( value[:-1] )
			data.append (new_row)
		upsert(pg_cur, table_name, selector_fields, setter_fields, data)
	except Exception as e: 
		traceback.print_exc()
		print (" Exception in IgnoreOrDeleteDoImportCodeDeck:  %s" % (str(e) ) )
		logger.error(" Exception in IgnoreOrDeleteDoImportCodeDeck:  %s" % (str(e) ) )
		DoError(row," Exception in IgnoreOrDeleteDoImportCodeDeck function:  %s" % (str(e)) )
		UpdateStatus(-5,row['id'])
		return [-2,"Error in Importing"]
	num_succ = len(csv_reader_list) - (num_fail+num_dubb)
	if some_error== False:
		UpdateStatus(2,row['id'],num_succ,num_fail,num_dubb)
	else:
		UpdateStatus(-1,row['id'],num_succ,num_fail,num_dubb)
		return [status,msg]
			
	return [0,"Done"]

	'''except Exception as e: 
		traceback.print_exc()
		print (" Exception in IgnoreOrDeleteDoImportCodeDeck:  %s" % (str(e) ) )
		logger.error(" Exception in IgnoreOrDeleteDoImportCodeDeck:  %s" % (str(e) ) )
		DoError(row," Exception in IgnoreOrDeleteDoImportCodeDeck function:  %s" % (str(e)) )
		UpdateStatus(-5,row['id'])
		return [-2,"Error in Importing"] '''


def IgnoreOrDeleteDoImportRoutePlan (scv_list,row,id_data,table_name,table_id_data,csv_reader_list) :
	print ("Entering IgnoreOrDeleteDoImportRoutePlan ")
	status = 0 
	msg = "Done"
	action =  row.get('duplicate_type',None)
	print ("action type is %s" % action)
	some_error= False
	num_succ  =0
	num_fail  =0
	num_dubb  =0
	route_strategy_id= row.get('foreign_id',None)
	if  route_strategy_id == None:
		UpdateStatus(-5,row['id'])
		return [-1,"foreign_id is missing. Abort!"]
	if action == None:
		action = 'delete'
	try:
		if action == 'ignore':  # loop over every row in csv to decide if the row to be included in insert or not
			counter= 1
			for csv_row in csv_reader_list:  # loop over all rows
				csv_row =stripSpace(csv_row)
				#ani_prefix	ani_min_length	ani_max_length	dnis_prefix	dnis_min_length	dnis_max_length	route_type	dynamic_route_name	static_route_name	intra_static_route_name	inter_static_route_name	
				# 0			1					2				3			4				5				6			7					8						9					10					
				
				if len(csv_row ) == 0:
					msg = " Possible empty rows:  %s  on row: %s" % (str(csv_row), counter    )
					DoError(row,msg,0) 
					#some_error= True
					num_fail = num_fail +1 
					counter = counter + 1
					continue
				
				ani_prefix = csv_row[0]
				dnis_prefix  = csv_row[3]
				ani_min_length = csv_row[1]
				if ani_min_length == "":
					ani_min_length= None
				else:
					ani_min_length = int(ani_min_length)
					if ani_min_length < 0:
						msg = "ani_min_length =0"
						DoError(row,msg,-1)
						print(msg)
						logger.debug(msg)
						some_error= True
						num_fail = num_fail +1 
						continue

				ani_max_length = csv_row[2]
				if ani_max_length == "":
					ani_max_length= None
				else:
					ani_max_length = int(ani_max_length)
					if ani_max_length < 0:
						msg = "ani_max_length =0"
						DoError(row,msg,-1)
						print(msg)
						logger.debug(msg)
						some_error= True
						num_fail = num_fail +1 
						continue

				dnis_min_length = csv_row[4]
				if dnis_min_length == "":
					dnis_min_length= None
				else:
					dnis_min_length = int(dnis_min_length)
					if dnis_min_length < 0:
						msg = "dnis_min_length =0"
						DoError(row,msg,-1)
						print(msg)
						logger.debug(msg)
						some_error= True
						num_fail = num_fail +1 
						continue
				
				dnis_max_length = csv_row[5]
				if dnis_max_length == "":
					dnis_max_length= None
				else:
					dnis_max_length = int(dnis_max_length)
					if dnis_max_length < 0:
						msg = "dnis_max_length =0"
						DoError(row,msg,-1)
						print(msg)
						logger.debug(msg)
						some_error= True
						num_fail = num_fail +1 
						continue
				
				#dynamic/static/static_jd/dynamic-static/dynamic-static_jd/static-dynamic/static-dynamic_jd
				route_type_dict= {
				'dynamic':1, 'static':2 , 'dynamic-static':3, 'static-dynamic':4 
				}
				route_type = csv_row[6]
				if route_type!="":
					route_type = route_type.strip()
					route_type =route_type_dict[route_type]
				else:
					route_type = None
				
				
				if route_type == None:
					msg = "Route type is not set"
					DoError(row,msg,-1)
					print(msg)
					logger.debug(msg)
					some_error= True
					num_fail = num_fail +1 
					continue
					
				dynamic_route_name = csv_row[7] if csv_row[7] != "" else None 
				static_route_name = csv_row[8] if csv_row[8] != "" else None  
				intra_static_route_name = csv_row[9] if csv_row[9] != "" else None 
				inter_static_route_name = csv_row[10] if csv_row[10] != "" else None 
				
				if static_route_name != None :
					sql = "select product_id as id from product where name = '%s'" %  static_route_name
					pg_cur.execute(sql)
					static_route_id = pg_cur.fetchone() 
					if static_route_id != None:
						static_route_id = static_route_id.get('id',None)
					
				else:
					static_route_id = None
				
				if dynamic_route_name != None :
					sql = "select dynamic_route_id  as id from dynamic_route  where name = '%s'" %  dynamic_route_name
					pg_cur.execute(sql)
					dynamic_route_id = pg_cur.fetchone() 
					if dynamic_route_id  != None:
						dynamic_route_id = dynamic_route_id .get('id',None)
					
				else:
					dynamic_route_id = None
				
				if intra_static_route_name != None :
					sql = "select product_id as id from product where name = '%s'" %  intra_static_route_name
					pg_cur.execute(sql)
					intra_static_route_id = pg_cur.fetchone() 
					if intra_static_route_id != None:
						intra_static_route_id = intra_static_route_id.get('id',None)
					
				else:
					intra_static_route_id = None
				
				if inter_static_route_name != None :
					sql = "select product_id as id from product where name = '%s'" %  inter_static_route_name
					pg_cur.execute(sql)
					inter_static_route_id = pg_cur.fetchone() 
					if inter_static_route_id != None:
						inter_static_route_id = inter_static_route_id.get('id',None)
					
				else:
					inter_static_route_id = None
				
				
				# route_strategy_id
				sql = """select count(*) as count from route  where ani_prefix=%s and 	digits= %s and route_strategy_id = %s and dynamic_route_id = %s"""   #% (csv_row[0],  foreign_id )
				print ( "SQL FOR IGNORE  ROUTE PLAN  option: %s"  %  sql )
				logger.debug( "SQL FOR IGNORE ROUTE PLAN option: %s"  %  sql )
				pg_cur.execute(sql,(ani_prefix,	dnis_prefix,route_strategy_id,dynamic_route_id))
				count= pg_cur.fetchone()[0]
				print ("count: ",count)
				if count >0 :
					some_error= True
					message = "Duplicate row:  %s" % str(csv_row)
					DoError(row,message,-1) 
					num_dubb = num_dubb +1
					continue
				counter = counter + 1
				sql = """INSERT INTO route  ( ani_prefix,ani_min_length,ani_max_length,digits,digits_min_length	,digits_max_length,route_type,dynamic_route_id,
				static_route_id,intra_static_route_id,inter_static_route_id,route_strategy_id ) VALUES (%s,%s,%s,%s,%s,%s,%s,%s,%s,%s,%s,%s)  """ 
				print ( "SQL FOR INSERT IN ROUTE: %s"  %  sql )
				logger.debug( "SQL FOR INSERT IN ROUTE: %s"  %  sql )
				pg_cur.execute(sql,(ani_prefix,	ani_min_length,	ani_max_length,	dnis_prefix,	dnis_min_length	,dnis_max_length,route_type,dynamic_route_id,
				static_route_id,intra_static_route_id,inter_static_route_id,route_strategy_id))
				num_succ = num_succ +1
			if some_error== False:
				UpdateStatus(2,row['id'],num_succ,num_fail,num_dubb)
			else:
				UpdateStatus(-1,row['id'],num_succ,num_fail,num_dubb)
			return [status,msg]
				
		else:
			counter= 1
			for csv_row in csv_reader_list:  # loop over all rows 
				#ani_prefix	ani_min_length	ani_max_length	dnis_prefix	dnis_min_length	dnis_max_length	route_type	dynamic_route_name	static_route_name	intra_static_route_name	inter_static_route_name	
				# 0			1					2				3			4				5				6			7					8						9					10					
				csv_row =stripSpace(csv_row)
				if len(csv_row ) == 0:
					msg = " Possible empty rows:  %s  on row: %s" % (str(csv_row), counter    )
					DoError(row,msg,0) 
					#some_error= True
					num_fail = num_fail +1 
					counter = counter + 1
					continue
				
				ani_prefix = csv_row[0]
				dnis_prefix  = csv_row[3]
				ani_min_length = csv_row[1]
				if ani_min_length == "":
					ani_min_length= None
				else:
					ani_min_length = int(ani_min_length)
					if ani_min_length < 0:
						msg = "ani_min_length =0"
						DoError(row,msg,-1)
						print(msg)
						logger.debug(msg)
						some_error= True
						num_fail = num_fail +1 
						continue

				ani_max_length = csv_row[2]
				if ani_max_length == "":
					ani_max_length= None
				else:
					ani_max_length = int(ani_max_length)
					if ani_max_length < 0:
						msg = "ani_max_length =0"
						DoError(row,msg,-1)
						print(msg)
						logger.debug(msg)
						some_error= True
						num_fail = num_fail +1 
						continue

				dnis_min_length = csv_row[4]
				if dnis_min_length == "":
					dnis_min_length= None
				else:
					dnis_min_length = int(dnis_min_length)
					if dnis_min_length < 0:
						msg = "dnis_min_length =0"
						DoError(row,msg,-1)
						print(msg)
						logger.debug(msg)
						some_error= True
						num_fail = num_fail +1 
						continue
				
				dnis_max_length = csv_row[5]
				if dnis_max_length == "":
					dnis_max_length= None
				else:
					dnis_max_length = int(dnis_max_length)
					if dnis_max_length < 0:
						msg = "dnis_max_length =0"
						DoError(row,msg,-1)
						print(msg)
						logger.debug(msg)
						some_error= True
						num_fail = num_fail +1 
						continue
				
				#dynamic/static/static_jd/dynamic-static/dynamic-static_jd/static-dynamic/static-dynamic_jd
				route_type_dict= {
				'dynamic':1, 'static':2 , 'dynamic-static':3, 'static-dynamic':4 
				}
				route_type = csv_row[6]
				if route_type!="":
					route_type = route_type.strip()
					route_type =route_type_dict[route_type]
				else:
					route_type = None
				
				
				if route_type == None:
					msg = "Route type is not set"
					DoError(row,msg,-1)
					print(msg)
					logger.debug(msg)
					some_error= True
					num_fail = num_fail +1 
					continue
					
				dynamic_route_name = csv_row[7] if csv_row[7] != "" else None 
				static_route_name = csv_row[8] if csv_row[8] != "" else None  
				intra_static_route_name = csv_row[9] if csv_row[9] != "" else None 
				inter_static_route_name = csv_row[10] if csv_row[10] != "" else None 
				
				if static_route_name != None :
					sql = "select product_id as id from product where name = '%s'" %  static_route_name
					pg_cur.execute(sql)
					static_route_id = pg_cur.fetchone() 
					if static_route_id != None:
						static_route_id = static_route_id.get('id',None)
					
				else:
					static_route_id = None
				
				if dynamic_route_name != None :
					sql = "select dynamic_route_id  as id from dynamic_route  where name = '%s'" %  dynamic_route_name
					pg_cur.execute(sql)
					dynamic_route_id = pg_cur.fetchone() 
					if dynamic_route_id != None:
						dynamic_route_id = dynamic_route_id.get('id',None)
					
				else:
					dynamic_route_id = None
				
				if intra_static_route_name != None :
					sql = "select product_id as id from product where name = '%s'" %  intra_static_route_name
					pg_cur.execute(sql)
					intra_static_route_id = pg_cur.fetchone() 
					if intra_static_route_id != None:
						intra_static_route_id = intra_static_route_id.get('id',None)
					
				else:
					intra_static_route_id = None
				
				if inter_static_route_name != None :
					sql = "select product_id as id from product where name = '%s'" %  inter_static_route_name
					pg_cur.execute(sql)
					inter_static_route_id = pg_cur.fetchone() 
					if inter_static_route_id != None:
						inter_static_route_id = inter_static_route_id.get('id',None)
					
				else:
					inter_static_route_id = None

				sql = """DELETE FROM  route  where ani_prefix=%s and digits= %s  and route_strategy_id = %s and dynamic_route_id = %s"""   #% (csv_row[0],  foreign_id )
				print ( "SQL FOR DELETE ROUTE PLAN option: %s"  %  sql )
				logger.debug( "SQL FOR DELETE ROUTE PLAN  option: %s"  %  sql )
				print ("ani_prefix,	dnis_prefix,route_strategy_id,dynamic_route_id", ani_prefix,dnis_prefix,route_strategy_id,dynamic_route_id)
				pg_cur.execute(sql,(ani_prefix,	dnis_prefix,route_strategy_id,dynamic_route_id))
				if pg_cur.rowcount > 0 :
					num_dubb = num_dubb +1
					num_succ = num_succ -1
				else:
					pass
				counter = counter + 1
				sql = """INSERT INTO route  ( ani_prefix,ani_min_length,ani_max_length,digits,digits_min_length	,digits_max_length,route_type,dynamic_route_id,
				static_route_id,intra_static_route_id,inter_static_route_id,route_strategy_id ) VALUES (%s,%s,%s,%s,%s,%s,%s,%s,%s,%s,%s,%s)  """ 
				print ( "SQL FOR INSERT IN ROUTE: %s"  %  sql )
				logger.debug( "SQL FOR INSERT IN ROUTE: %s"  %  sql )
				pg_cur.execute(sql,(ani_prefix,	ani_min_length,	ani_max_length,	dnis_prefix,	dnis_min_length	,dnis_max_length,route_type,dynamic_route_id,
				static_route_id,intra_static_route_id,inter_static_route_id,route_strategy_id))
				num_succ = num_succ +1
				

			if some_error== False:
				UpdateStatus(2,row['id'],num_succ,num_fail,num_dubb)
			else:
				UpdateStatus(-1,row['id'],num_succ,num_fail,num_dubb)
			return [status,msg]
			
		return [0,"Done"]
	except Exception as e: 
		print (" Exception in IgnoreOrDeleteDoImportRoutePlan:  %s" % (str(e) ) )
		logger.error(" Exception in IgnoreOrDeleteDoImportRoutePlan:  %s" % (str(e) ) )
		DoError(row," Exception in IgnoreOrDeleteDoImportRoutePlan function:  %s" % (str(e)) )
		UpdateStatus(-5,row['id'])
		return [-2,"Error in Importing"]



def IgnoreOrDeleteDoImportStaticRoutePlan (scv_list,row,id_data,table_name,table_id_data,csv_reader_list) :
	print ("Entering IgnoreOrDeleteDoImportStaticRoutePlan ")
	status = 0 
	msg = "Done"
	action =  row.get('duplicate_type',None)
	print ("action type is %s" % action)
	some_error= False
	num_succ  =0
	num_fail  =0
	num_dubb  =0
	product_id= row.get('foreign_id',None)
	if  product_id == None:
		UpdateStatus(-5,row['id'])
		DoError(row,"foreign_id is missing",-5)
		return [-1,"foreign_id is missing. Abort!"]
	#sql = "select * from product where product_id= %s"
	#pg_cur.execute(sql,(product_id))
	#if pg_cur.fetchone() == None:
	#	DoError(row,"static route id %s_product not exist"% product_id,-5)
	#	print ("static route id %s_product not exist"% product_id)
	#	return [-1,"static route id %s_product not exist"% product_id]
		
	if action == None:
		action = 'delete'
	try:
		
		counter= 1
		for csv_row in csv_reader_list:  # loop over all rows 
			csv_row =stripSpace(csv_row)
			if len(csv_row ) == 0:
				msg = " Possible empty rows: %s  on row Number: %s" % (str(csv_row), counter    )
				DoError(row,msg,0) 
				#some_error= True
				num_fail = num_fail +1 
				counter = counter + 1
				continue
			
			digits = csv_row[0] 
			stratagy_name = csv_row[1] if csv_row[1] != "" else None 
			trunk_name = csv_row[2] if csv_row[2] != "" else None  
			time_profile_name = csv_row[4] if csv_row[4] != "" else None 
			percentage = csv_row[3] if csv_row[3] != "" else None 
			
		# strategy   0 By Percentage 1: Top-Down  2:Round-Robin round-robin/top-down/percentage
			stratagy_table = {'percentage':0,'top-down':1, 'round-robin':2,None:None}
			try:
				stratagy_id = stratagy_table [stratagy_name]
			except :
				print ("possible header. skip!")
				continue
			
			#select (select name from time_profile where time_profile_id = product_items.time_profile_id) as time_profile from product_items where product_id = '22' ;
			#select  name,time_profile_id from time_profile  ;
			if time_profile_name != None :
				sql = "select time_profile_id as id from time_profile where name = '%s'" %  time_profile_name
				pg_cur.execute(sql)
				time_profile_id= pg_cur.fetchone() 
				print("time_profile_id q",str(time_profile_id))
				if time_profile_id != None:
					time_profile_id = time_profile_id.get('id',None)
				
			else:
				time_profile_id = None
			
			print("trunk_name" ,trunk_name)
			#select alias ,resource_id from resource where alias = 'rere'  ;
			#select item_id   from product_items_resource  where product_items_resource.resource_id  = 79;
			# select item_id   from product_items_resource  where  
			# left join product_items_resource on resource.resource_id = product_items_resource.resource_id where product_items_resource.item_id =product_items.item_id
			if trunk_name != None :
				sql = "select resource_id  as id from resource  where alias = '%s'" %  trunk_name
				pg_cur.execute(sql)
				resource_id = pg_cur.fetchone() 
				if resource_id != None:
					resource_id= resource_id.get('id',None)
				else:
					DoError(row,"Resource name invalid or missing",-1)
					num_fail = num_fail + 1
			else:
				resource_id= None
			
			print ("resource_id",resource_id)
			
			# select item_id,by_percentage,resource_id    from product_items_resource 
			if resource_id == None:
				DoError(row,"Resource name invalid or missing",-1)
				num_fail = num_fail + 1
				continue
			if action == 'ignore':  # loop over every row in csv to decide if the row to be included in insert or not
			# product_id
				sql = """select count(*) as count from  product_items,  product_items_resource where   product_items_resource.item_id = product_items.item_id and
				 digits= %s and resource_id = %s """   #% (csv_row[0],  foreign_id )
				print ( "SQL FOR IGNORE STATIC ROUTE PLAN  option: %s"  %  sql )
				logger.debug( "SQL FOR IGNORE STATIC ROUTE PLAN option: %s"  %  sql )
				pg_cur.execute(sql,(digits,resource_id))
				count= pg_cur.fetchone()[0]
				print ("count: ",count)
				if count >0 :
					message = "Duplicate row:  %s" % str(csv_row)
					DoError(row,message,-1) 
					num_dubb = num_dubb +1
					continue
				counter = counter + 1
				sql = """INSERT INTO product_items  ( digits ,product_id , time_profile_id,strategy, update_by
				 ) VALUES (%s,%s,%s,%s,%s)  returning item_id""" 
				print ( "SQL FOR INSERT IN STATIC ROUTE: %s"  %  sql )
				logger.debug( "SQL FOR INSERT IN STATIC ROUTE: %s"  %  sql )
				pg_cur.execute(sql,(digits ,product_id , time_profile_id,stratagy_id, "Import script"))
				item_id = pg_cur.fetchone() [0]
				sql = """INSERT INTO product_items_resource ( item_id,resource_id,by_percentage )  VALUES (%s,%s,%s) """
				pg_cur.execute(sql,(item_id,resource_id,percentage ))
				num_succ = num_succ +1
				
			else:
				sql = """select product_items.item_id as item_id from  product_items,  product_items_resource where   product_items_resource.item_id = product_items.item_id and
				 digits= %s and resource_id = %s """   #% (csv_row[0],  foreign_id )
				print(sql)
				pg_cur.execute(sql,(digits,resource_id))
				item_id= pg_cur.fetchone()
				print("i1",item_id)
				if item_id != None:
					
					print("item_id1",item_id)
					item_id=item_id[0]
					message = "Duplicate row:  %s" % str(csv_row)
					DoError(row,message,-1) 
					num_dubb = num_dubb +1
					sql = """DELETE FROM  product_items where item_id= %s""" %item_id  #% (csv_row[0],  foreign_id )
					pg_cur.execute(sql)
					sql = """DELETE FROM   product_items_resource where item_id= %s"""%item_id   #% (csv_row[0],  foreign_id )
					pg_cur.execute(sql )
				counter = counter + 1
				sql = """INSERT INTO product_items  ( digits ,product_id , time_profile_id,strategy, update_by
				 ) VALUES (%s,%s,%s,%s,%s)  returning item_id""" 
				print ( "SQL FOR INSERT IN STATIC ROUTE: %s"  %  sql )
				logger.debug( "SQL FOR INSERT IN STATIC ROUTE: %s"  %  sql )
				pg_cur.execute(sql,(digits ,product_id , time_profile_id,stratagy_id, "Import script"))
				item_id = pg_cur.fetchone()[0] # [0]
				print("item_id2",item_id)
				sql = """INSERT INTO product_items_resource ( item_id,resource_id,by_percentage )  VALUES (%s,%s,%s) """
				pg_cur.execute(sql,(item_id,resource_id,percentage ))
				num_succ = num_succ +1
				
		if some_error== False:
			UpdateStatus(2,row['id'],num_succ,num_fail,num_dubb)
		else:
			UpdateStatus(-1,row['id'],num_succ,num_fail,num_dubb)
		return [status,msg]
			
	except Exception as e: 
		print (" Exception in IgnoreOrDeleteDoImportStaticRoutePlan:  %s" % (str(e) ) )
		logger.error(" Exception in IgnoreOrDeleteDoImportStaticRoutePlan:  %s" % (str(e) ) )
		DoError(row," Exception in IgnoreOrDeleteDoImportStaticRoutePlan function:  %s" % (str(e)) )
		import traceback
		traceback.print_exc()
		UpdateStatus(-5,row['id'])
		return [-2,"Error in Importing"]


def IgnoreOrDeleteDoImportEgress (scv_list,row,id_data,table_name,table_id_data,csv_reader_list) :
	print ("Entering IgnoreOrDeleteDoImportEgress ")
	status = 0 
	msg = "Done"
	action =  row.get('duplicate_type',None)
	print ("action type is %s" % action)
	some_error= False
	num_succ  =0
	num_fail  =0
	num_dubb  =0
	#foreign_id= row.get('foreign_id',None)
	#if  product_id == None:
	#	UpdateStatus(-5,row['id'])
	#	DoError(row,"foreign_id is missing",-5)
	#	return [-1,"foreign_id is missing. Abort!"]
	
	if action == None:
		action = 'delete'
	try:
		#trunk_name	carrier_name	media_type	call_limit	cps_limit	pdd_timeout	active		rate_table_name	host_route_strategy		rfc2833		pass_dip_head	min_duration	max_duration
		#egress_001	client_002		bypass/proxy	1				1	1000		true/false	rate_table_001	round-robin/top-down	true/false	true/false			1			1
		#0				1			2				3				4	5			6				7				8						9			10				11			12
		counter= 1
		if len(csv_reader_list) < 10 :
			print ("Egress csv_reader_list" ,csv_reader_list)
		for csv_row in csv_reader_list:  # loop over all rows 
			csv_row =stripSpace(csv_row)
			if len(csv_row ) == 0:
				msg = " Possible empty rows:  %s  on row: %s" % (str(csv_row), counter    )
				DoError(row,msg,0) 
				#some_error= True
				num_fail = num_fail +1 
				counter = counter + 1
				continue
			
			carrier_name = csv_row[1] if csv_row[1] != "" else None 
			media_type = 1 if csv_row[2].lower() == "proxy"  else 1 if   csv_row[2].lower() == "transcoding" else 2
			res_strategy = 1 if csv_row[8].lower() == "top-down"   else 2 
			trunk_name = csv_row[0] if csv_row[0] != "" else None    #resource.alias
			capacity = csv_row[3] if csv_row[3] != "" else None   #call limit
			cps_limit = csv_row[4] if csv_row[4] != "" else None 
			wait_ringtime180 =csv_row[5] if csv_row[5] != "" else None  #pdd timeout
			_active = False if csv_row[6].lower() == "false" else True 
			rate_table_name = csv_row[7] if csv_row[7] != "" else None 
			rfc2833 =True if csv_row[9].lower() == "true" else False
			pass_dip_head = True if csv_row[10].lower() == "true" else False   # Jack: pass_dip_head   =  lnp_dipping in DB field
			delay_bye_second= csv_row[11] if csv_row[11] != "" else None 
			max_duration = csv_row[12] if csv_row[12] != "" else None
			print ("wait_ringtime180",wait_ringtime180)
			
			#wait_ringtime180 = pdd ; max_duration= Max duration ; delay_bye_second= Min duration ;

			# check up for min and max duration
			try:
				delay_bye_second = int(delay_bye_second)
				max_duration = int(max_duration)
				'''if min_duration < 1 or max_duration > 32 or min_duration > max_duration:
					DoError(row,"for row %s min and max durations are not setup properly"% counter,-4)
					num_fail = num_fail + 1
					counter = counter + 1 
					continue'''
			except:
					delay_bye_second = None
					max_duration = None
			
			if carrier_name== None and  trunk_name== None:
				DoError(row,"Wrong or empty row %s:  on line %s "% (str(csv_row),counter),-4)
				num_fail = num_fail + 1
				counter = counter + 1 
				continue
			
			
			
			
			if capacity == None:
				try:
					capacity = int(capacity)
				except:
					capacity = None
					
			if cps_limit == None:
				try:
					cps_limit = int(cps_limit)
				except:
					cps_limit = None
			
			if wait_ringtime180 != None:
				try:
					wait_ringtime180 = int(wait_ringtime180)
				except:
					print ("Must be integer")
					wait_ringtime180 = 6001
			else:
				print ("wait_ringtime180 is none" )
				wait_ringtime180 = 0
				
			print ("wait_ringtime180",wait_ringtime180)
			if rate_table_name != None :
				sql = "select rate_table_id as id from rate_table where name = '%s'" %  rate_table_name
				pg_cur.execute(sql)
				rate_table_id= pg_cur.fetchone() 
				print("rate_table_id",str(rate_table_id))
				if rate_table_id != None:
					rate_table_id = rate_table_id.get('id',None)
					
			else:
				rate_table_id = None
				
			if rate_table_id == None:
				DoError(row,"Rate table name not exists %s"% counter,-4)
				num_fail = num_fail + 1
				counter = counter + 1 
				continue
				
			sql = """select client_id from client where name = '%s'""" % carrier_name
			pg_cur.execute(sql)
			client_id= pg_cur.fetchone()
			
			print ("client_id",client_id)
			if client_id== None:
				DoError(row,"Not exisiting client name/id for row %s"% counter,-4)
				num_fail = num_fail + 1
				counter = counter + 1 
				continue
			else:
				client_id= client_id['client_id']
				
			if action == 'ignore':  # loop over every row in csv to decide if the row to be included in insert or not
			# product_id
				#sql = """select count(*) as count from resource where alias= '%s' """ %trunk_name  #% (csv_row[0],  foreign_id )
				#print ( "SQL FOR IGNORE EGRESS option: %s"  %  sql )
				#logger.debug( "SQL FOR IGNORE EGRESS option: %s"  %  sql )
				#print ("trunk_name", trunk_name)
				#pg_cur.execute(sql)
				#count= pg_cur.fetchone()[0]
				#print ("count: ",count)
				#if count >0 :
				#	some_error= True
				#	message = "Duplicate row:  %s" % str(csv_row)
				#	DoError(row,message,-1) 
				#	num_dubb = num_dubb +1
				#	continue
				counter = counter + 1
				sql = """INSERT INTO resource   ( alias , cps_limit ,update_at ,  update_by, capacity , egress , active , wait_ringtime180,   rate_table_id ,lnp_dipping,client_id
				,delay_bye_second,max_duration ) VALUES (%s,%s, current_timestamp(0)  ,'Import Script',%s,%s,%s,%s,%s,%s,%s,%s,%s)  returning resource_id""" 
				print ( "SQL FOR INSERT EGRESS: %s"  %  sql )
				logger.debug( "SQL FOR INSERT IN EGRESS: %s"  %  sql )
				try:
					pg_cur.execute(sql,( trunk_name, cps_limit,capacity ,True, _active,wait_ringtime180,  rate_table_id ,pass_dip_head,client_id,delay_bye_second,max_duration))
				except Exception as e: 
					message = "Duplicate row:  %s" % str(csv_row)
					DoError(row,message,-1) 
					num_dubb = num_dubb +1
					continue
				
				resource_id = pg_cur.fetchone() [0]
				print ("New resource_id: %s"% resource_id)
				num_succ = num_succ +1
				
			else:
				sql = """DELETE FROM  resource where alias= '%s'""" %trunk_name   #% (csv_row[0],  foreign_id )
				print ( "SQL FOR DELETE EGRESS option: %s"  %  sql )
				logger.debug( "SQL FOR EGRESS option: %s"  %  sql )
				pg_cur.execute(sql)
				if pg_cur.rowcount > 0 :
					num_dubb = num_dubb +1
					num_succ = num_succ -1
				else:
					pass
				counter = counter + 1
				sql = """INSERT INTO resource   ( alias , cps_limit ,update_at ,  update_by, capacity , egress , active , wait_ringtime180,   rate_table_id ,lnp_dipping,client_id,
				 delay_bye_second,max_duration) VALUES (%s,%s, current_timestamp(0)  ,'Import Script',%s,%s,%s,%s,%s,%s,%s,%s,%s)  returning resource_id""" 
				print ( "SQL FOR INSERT EGRESS: %s"  %  sql )
				logger.debug( "SQL FOR INSERT IN EGRESS: %s"  %  sql )
				pg_cur.execute(sql,( trunk_name, cps_limit,capacity ,True, _active,wait_ringtime180,  rate_table_id ,pass_dip_head,client_id,delay_bye_second,max_duration))
				resource_id = pg_cur.fetchone() [0]
				print ("New resource_id: %s"% resource_id)
				num_succ = num_succ +1
				
		if some_error== False:
			UpdateStatus(2,row['id'],num_succ,num_fail,num_dubb)
		else:
			UpdateStatus(-1,row['id'],num_succ,num_fail,num_dubb)
		return [status,msg]
			
	except Exception as e: 
		print (" Exception in IgnoreOrDeleteDoImportEgress:  %s" % (str(e) ) )
		logger.error(" Exception in IgnoreOrDeleteDoImportEgress:  %s" % (str(e) ) )
		DoError(row," Exception in IgnoreOrDeleteDoImportEgress function:  %s" % (str(e)) )
		traceback.print_exc()
		UpdateStatus(-5,row['id'])
		return [-2,"Error in Importing"]


def IgnoreOrDeleteDoImportIngress (scv_list,row,id_data,table_name,table_id_data,csv_reader_list) :
	print ("Entering IgnoreOrDeleteDoImportIngress ")
	status = 0 
	msg = "Done"
	action =  row.get('duplicate_type',None)
	print ("action type is %s" % action)
	some_error= False
	num_succ  =0
	num_fail  =0
	num_dubb  =0
	#foreign_id= row.get('foreign_id',None)
	#if  product_id == None:
	#	UpdateStatus(-5,row['id'])
	#	DoError(row,"foreign_id is missing",-5)
	#	return [-1,"foreign_id is missing. Abort!"]
	
	if action == None:
		action = 'delete'
	try:
		#trunk_name		carrier_name	media_type		cps_limit	call_limit	pdd_timeout		ignore_early_media				active		rfc2833			dip_from		min_duration	max_duration
		#ingress_001	client_001		bypass/proxy		1			1			1000		none/180/183/180 and 183	true/false		true/false		client/server		1				1
		#0				1					2				3			4			5					6						7				8				9				10				11			
		counter= 1
		for csv_row in csv_reader_list:  # loop over all rows 
			csv_row =stripSpace(csv_row)
			
			if len(csv_row ) == 0:
				msg = " Possible empty rows:  %s  on row: %s" % (str(csv_row), counter    )
				DoError(row,msg,0) 
				#some_error= True
				num_fail = num_fail +1 
				counter = counter + 1
				continue
			
			carrier_name = csv_row[1] if csv_row[1] != "" else None 
			media_type = 1 if csv_row[2].lower() == "proxy"  else 1 if   csv_row[2].lower() == "transcoding" else 2
			res_strategy = 1 if csv_row[8].lower() == "top-down"   else 2 
			trunk_name = csv_row[0] if csv_row[0] != "" else None    #resource.alias
			capacity = csv_row[4] if csv_row[4] != "" else None   #call limit
			cps_limit = csv_row[3] if csv_row[3] != "" else None 
			wait_ringtime180 =csv_row[5] if csv_row[5] != "" else None  #pdd timeout
			_active = False if csv_row[7].lower() == "false" else True 
			ignore_early_media_str = csv_row[6] if csv_row[6] != "" else None   
			rfc2833 =True if csv_row[8].lower() == "true" else False
			dip_from = csv_row[9] if csv_row[9] != "" else None 
			min_duration = csv_row[10] if csv_row[10] != "" else None 
			max_duration = csv_row[11] if csv_row[11] != "" else None 
			
			if ignore_early_media_str != None:  #'ignore_early_media can be : none,180,183,180 and 183',
				if ignore_early_media_str.strip() == '180':  # resource.ignore_early_media is bool!
					ignore_ring = True
					ignore_early_media = False
				elif ignore_early_media_str.strip() == '100':
					ignore_ring = False
					ignore_early_media = True
				
				elif ignore_early_media_str.strip() == '180 and 183':
					ignore_ring = True
					ignore_early_media = True
				
				else:
					print ('ignore_early_media can be : none,180,183,180 and 183')
					DoError(row,"for row %s ignore_early_media can be : none,180,183,180 and 183"% counter,-2)
					num_fail = num_fail + 1
					counter = counter + 1 
					some_error= True
					continue
				
			
			else:
				ignore_early_media = False
				ignore_ring = False
			
			
			# check up for min and max duration
			'''
			try:
				min_duration = int(min_duration)
				max_duration = int(max_duration)
				if min_duration < 1 or max_duration > 32 or min_duration > max_duration:
					DoError(row,"for row %s min and max durations are not setup properly"% counter,-4)
					num_fail = num_fail + 1
					counter = counter + 1 
					continue
			except:
					DoError(row,"for row %s min and max durations are not setup properly"% counter,-4)
					num_fail = num_fail + 1
					counter = counter + 1 
					some_error= True
					continue'''
			
			if carrier_name== None :
				DoError(row,"Missing trunk or carrier name for row %s"% counter,-4)
				num_fail = num_fail + 1
				counter = counter + 1 
				continue
			
			
			if capacity == None:
				try:
					capacity = int(capacity)
				except:
					capacity = None
					
			if cps_limit == None:
				try:
					cps_limit = int(cps_limit)
				except:
					cps_limit = None
			
			if wait_ringtime180 != None:
				try:
					wait_ringtime180 = int(wait_ringtime180)
				except:
					wait_ringtime180 = 6000
			else:
				wait_ringtime180 = 6000

				
			sql = """select client_id from client where name = '%s'""" % carrier_name
			pg_cur.execute(sql)
			client_id= pg_cur.fetchone()
			
			print ("client_id",client_id)
			if client_id== None:
				DoError(row,"Not exisiting client name/id for row %s"% counter,-4)
				num_fail = num_fail + 1
				counter = counter + 1 
				continue
			else:
				client_id= client_id['client_id']
				
			if action == 'ignore':  # loop over every row in csv to decide if the row to be included in insert or not
			# product_id

				counter = counter + 1
				sql = """INSERT INTO resource   ( alias , cps_limit ,update_at ,  update_by, capacity , ingress , active , wait_ringtime180,  client_id,ignore_ring,ignore_early_media
				,delay_bye_second,max_duration  ) VALUES (%s,%s, current_timestamp(0)  ,'Import Script',%s,%s,%s,%s,%s,%s,%s,%s,%s)  returning resource_id""" 
				print ( "SQL FOR INSERT INGRESS: %s"  %  sql )
				logger.debug( "SQL FOR INSERT IN INGRESS: %s"  %  sql )
				try:
					pg_cur.execute(sql,( trunk_name, cps_limit,capacity ,True, _active,wait_ringtime180,  client_id,ignore_ring,ignore_early_media,min_duration,max_duration))
				except Exception as e: 
					message = "Error:  %s" % str(csv_row)
					DoError(row,message,-1) 
					num_fail = num_fail +1
					continue
				
				resource_id = pg_cur.fetchone() [0]
				print ("New resource_id: %s"% resource_id)
				num_succ = num_succ +1
				
			else:
				sql = """DELETE FROM  resource where alias= '%s'""" %trunk_name   #% (csv_row[0],  foreign_id )
				print ( "SQL FOR DELETE EGRESS option: %s"  %  sql )
				logger.debug( "SQL FOR EGRESS option: %s"  %  sql )
				pg_cur.execute(sql)
				if pg_cur.rowcount > 0 :
					num_dubb = num_dubb +1
					num_succ = num_succ -1
				else:
					pass
				counter = counter + 1
				sql = """INSERT INTO resource   ( alias , cps_limit ,update_at ,  update_by, capacity , ingress , active , wait_ringtime180,  client_id,ignore_ring,ignore_early_media
				,delay_bye_second,max_duration  ) VALUES (%s,%s, current_timestamp(0)  ,'Import Script',%s,%s,%s,%s,%s,%s,%s,%s,%s)  returning resource_id""" 
				print ( "SQL FOR INSERT INGRESS: %s"  %  sql )
				logger.debug( "SQL FOR INSERT IN INGRESS: %s"  %  sql )
				try:
					pg_cur.execute(sql,( trunk_name, cps_limit,capacity ,True, _active,wait_ringtime180,  client_id,ignore_ring,ignore_early_media,min_duration,max_duration))
				except Exception as e: 
					message = "Error:  %s" % str(csv_row)
					DoError(row,message,-1) 
					num_fail = num_fail +1
					continue
				resource_id = pg_cur.fetchone() [0]
				print ("New resource_id: %s"% resource_id)
				num_succ = num_succ +1
				
		if some_error== False:
			UpdateStatus(2,row['id'],num_succ,num_fail,num_dubb)
		else:
			UpdateStatus(-1,row['id'],num_succ,num_fail,num_dubb)
		return [status,msg]
			
	except Exception as e: 
		print (" Exception in IgnoreOrDeleteDoImportIngress:  %s" % (str(e) ) )
		logger.error(" Exception in IgnoreOrDeleteDoImportIngress:  %s" % (str(e) ) )
		DoError(row," Exception in IgnoreOrDeleteDoImportIngress function:  %s" % (str(e)) )
		traceback.print_exc()
		UpdateStatus(-5,row['id'])
		return [-2,"Error in Importing"]







def IgnoreOrDeleteDoImportHost (scv_list,row,id_data,table_name,table_id_data,csv_reader_list) :
	print ("Entering IgnoreOrDeleteDoImportHost ")
	status = 0 
	msg = "Done"
	action =  row.get('duplicate_type',None)
	print ("action type is %s" % action)
	some_error= False
	num_succ  =0
	num_fail  =0
	num_dubb  =0
	foreign_id= row.get('foreign_id',0)
	#if  product_id == None:
	
	import socket
	if action == None:
		action = 'delete'
	try:
		#trunk_name	ip	port
		#0			1	2	
		counter= 1
		for csv_row in csv_reader_list:  # loop over all rows 
			csv_row =stripSpace(csv_row)
			if len(csv_row ) == 0:
				msg = " Possible empty rows:  %s  on row: %s" % (str(csv_row), counter    )
				DoError(row,msg,0) 
				#some_error= True
				num_fail = num_fail +1 
				counter = counter + 1
				continue
			
			ip = csv_row[1] if csv_row[1] != "" else None 
			port = csv_row[2] if csv_row[2] != ""  else None
			trunk_name = csv_row[0] if csv_row[0] != "" else None    #resource.alias
			
			if ip == None and port == None and trunk_name== None:
				print ('Empty row')
				DoError(row,"Empty row %s "% counter,0)
				num_fail = num_fail + 1
				counter = counter + 1 
				continue
			
			if ip == None:  #'ignore_early_media can be : none,180,183,180 and 183',
				print ('IP is not set')
				DoError(row,"for row %s IP is not set correctly"% counter,-4)
				num_fail = num_fail + 1
				counter = counter + 1 
				continue
			
			
			try: 
				socket.inet_aton(str( ip) )
			except:
				print ('Invalid IP')
				DoError(row,"for row %s Invalid IP"% counter,-4)
				num_fail = num_fail + 1
				counter = counter + 1 
				continue
			
			
			if port == None:  #'ignore_early_media can be : none,180,183,180 and 183',
				print ('Port is not set')
				DoError(row,"for row %s Port is not set correctly"% counter,-4)
				num_fail = num_fail + 1
				counter = counter + 1 
				continue
			
			
			if  trunk_name== None:
				DoError(row,"Missing trunk  name for row %s"% counter,-4)
				num_fail = num_fail + 1
				counter = counter + 1 
				continue
			
			sql = """select resource_id from resource where alias = '%s'""" % trunk_name
			pg_cur.execute(sql)
			resource_id = pg_cur.fetchone()
			if resource_id == None:
				print ("Invalid trunk name for row %s"%counter)
				DoError(row,"Invalid trunk name for row %s"%counter,-4)
				num_fail = num_fail + 1
				counter = counter + 1 
				continue
			
			resource_id = resource_id ['resource_id']
			
			if action == 'ignore':  # loop over every row in csv to decide if the row to be included in insert or not
			# product_id
				sql = """select count(*) as count from resource_ip where resource_id= %s and ip= '%s' """ %( resource_id, ip )
				print ( "SQL FOR IGNORE HOST option: %s"  %  sql )
				logger.debug( "SQL FOR IGNORE HOST option: %s"  %  sql )
				print ("resource_id", resource_id)
				pg_cur.execute(sql)
				count= pg_cur.fetchone()[0]
				print ("count: ",count)
				if count >0 :
					some_error= True
					message = "Duplicate row:  %s" % str(csv_row)
					DoError(row,message,-1) 
					num_dubb = num_dubb +1
					continue
				counter = counter + 1
				sql = """INSERT INTO resource_ip  (resource_id, ip,port ) VALUES (%s,%s,%s)  """ 
				print ( "SQL FOR INSERT HOST: %s"  %  sql )
				logger.debug( "SQL FOR INSERT IN HOST: %s"  %  sql )
				pg_cur.execute(sql,( resource_id, ip,port))
				num_succ = num_succ +1
				
			else:
				sql = """DELETE FROM resource_ip where resource_id= %s and ip = '%s'""" %(resource_id,ip )   #% (csv_row[0],  foreign_id )
				print ( "SQL FOR DELETE HOSToption: %s"  %  sql )
				logger.debug( "SQL FOR HOST option: %s"  %  sql )
				pg_cur.execute(sql)
				if pg_cur.rowcount > 0 :
					num_dubb = num_dubb +1
					num_succ = num_succ -1
				else:
					pass
				counter = counter + 1
				sql = """INSERT INTO resource_ip  (resource_id, ip,port ) VALUES (%s,%s,%s) """ 
				print ( "SQL FOR INSERT HOST: %s"  %  sql )
				logger.debug( "SQL FOR INSERT IN HOST: %s"  %  sql )
				pg_cur.execute(sql,( resource_id, ip,port))
				num_succ = num_succ +1
				
		if some_error== False:
			UpdateStatus(2,row['id'],num_succ,num_fail,num_dubb)
		else:
			UpdateStatus(-1,row['id'],num_succ,num_fail,num_dubb)
		return [status,msg]
			
	except Exception as e: 
		print (" Exception in IgnoreOrDeleteDoImportHost:  %s" % (str(e) ) )
		logger.error(" Exception in IgnoreOrDeleteDoImportHost:  %s" % (str(e) ) )
		DoError(row," Exception in IgnoreOrDeleteDoImportHost function:  %s" % (str(e)) )
		traceback.print_exc()
		UpdateStatus(-5,row['id'])
		return [-2,"Error in Importing"]



def IgnoreOrDeleteDoImportDigitMapping (scv_list,row,id_data,table_name,table_id_data,csv_reader_list) :
	print ("Entering IgnoreOrDeleteDoImportDigitMapping ")
	status = 0 
	msg = "Done"
	action =  row.get('duplicate_type',None)
	print ("action type is %s" % action)
	some_error= False
	num_succ  =0
	num_fail  =0
	num_dubb  =0
	translation_id= row.get('foreign_id',0)
	if  translation_id == None:
		debug.error("foreign_id is missing from DB")
		DoError(row,"foreign_id is missing from DB")
		return 
	
	if action == None:
		action = 'delete'
	try:
		#ani_method										ani						translated_ani				dnis_method								dnis	translated_dnis
		#ignore/replace	matched prefix/replace			1							2						ignore/replace matched prefix/replace	1			2
		#0						1						2							3								4								5			6
		counter= 1
		for csv_row in csv_reader_list:  # loop over all rows 
			#check for empty row:
			csv_row =stripSpace(csv_row)
			if len(csv_row ) == 0:
				msg = " Possible empty rows:  %s  on row: %s" % (str(csv_row), counter    )
				DoError(row,msg,0) 
				#some_error= True
				num_fail = num_fail +1 
				counter = counter + 1
				continue

			if csv_row[0] == "":
				is_empty = all(csv_row[0] == item for item in csv_row)
				if is_empty:  # row is empty!
					msg = " Possible empty rows:  %s  on row: %s" % (str(csv_row), counter    )
					DoError(row,msg,0) 
					num_fail = num_fail +1 
					counter = counter + 1
					continue

			
			print("Debug Digit trance:",  str(csv_row),len(csv_row)  )
			ani_method_str= str(csv_row[0]) if csv_row[0] != "" else None 
			ani= csv_row[1] if csv_row[1] != ""  else None
			translated_ani = csv_row[2] if csv_row[2] != "" else None   
			dnis_method_str= csv_row[3] if csv_row[3] != "" else None 
			dnis= csv_row[4] if csv_row[4] != ""  else None
			translated_dnis = csv_row[5] if csv_row[5] != "" else None 
			
			if ani_method_str == None:  #'ignore_early_media can be : none,180,183,180 and 183',
				ani_method = 0 
			else:
				ani_method = 0 if ani_method_str.lower().strip() == "ignore" else 1 if ani_method_str.lower().strip() == "replace matched prefix" else 2
			dnis_method = 0 if dnis_method_str.lower().strip() == "ignore" else 1 if dnis_method_str.lower().strip() == "replace matched prefix" else 2
			
			try: 
				ani = int(ani)
				translated_ani = int(translated_ani)
			except:
				ani = None
				translated_ani = None
			try: 
				dnis = int(dnis)
				translated_dnis = int(translated_dnis)
			except:
				dnis = None
				translated_dnis = None
				
			if (ani == None and dnis == None):
				DoError(row,"for row №%s Ani and Dnis cannot be null both!"% counter,-4)
				num_fail = num_fail + 1
				counter = counter + 1 
				continue
				
			
			if action == 'ignore':  # loop over every row in csv to decide if the row to be included in insert or not
			# product_id
				sql = """select count(*) as count from translation_item where translation_id = %s and ani = %s and dnis = %s"""  #% (csv_row[0],  foreign_id )
				print ( "SQL FOR IGNORE DigitMapping option: %s"  %  sql )
				logger.debug( "SQL FOR IGNORE DigitMapping option: %s"  %  sql )
				pg_cur.execute(sql,(translation_id, ani,dnis) )
				count= pg_cur.fetchone()[0]
				print ("count: ",count)
				if count >0 :
					some_error= True
					message = "Duplicate row:  %s" % str(csv_row)
					DoError(row,message,-1) 
					num_dubb = num_dubb +1
					continue
				counter = counter + 1
				sql = """INSERT INTO translation_item (translation_id,ani_method, dnis_method, ani,action_ani,dnis ,action_dnis) VALUES (%s,%s,%s,'%s',%s,'%s',%s)  """ 
				print ( "SQL FOR INSERT DigitMapping: %s"  %  sql )
				logger.debug( "SQL FOR INSERT IN DigitMapping: %s"  %  sql )
				pg_cur.execute(sql,( translation_id,ani_method, dnis_method, ani,translated_ani,dnis ,translated_dnis))
				num_succ = num_succ +1
				
			else:
				
				sql = """DELETE FROM translation_item where translation_id = %s and ani = %s and dnis = %s"""   #% (csv_row[0],  foreign_id )
				print ( "SQL FOR DELETE DigitMapping option: %s"  %  sql )
				logger.debug( "SQL FOR DigitMapping option: %s"  %  sql )
				pg_cur.execute(sql,(translation_id, str(ani),str(dnis)) )
				if pg_cur.rowcount > 0 :
					num_dubb = num_dubb +1
					num_succ = num_succ -1
				else:
					pass
				counter = counter + 1
				sql = """INSERT INTO translation_item (translation_id,ani_method, dnis_method, ani,action_ani,dnis ,action_dnis) VALUES (%s,%s,%s,'%s',%s,'%s',%s)  """ 
				print ( "SQL FOR INSERT DigitMapping: %s"  %  sql )
				logger.debug( "SQL FOR INSERT IN DigitMapping: %s"  %  sql )
				pg_cur.execute(sql,( translation_id,ani_method, dnis_method, ani,translated_ani,dnis ,translated_dnis))
				num_succ = num_succ +1
				
		if some_error== False:
			UpdateStatus(2,row['id'],num_succ,num_fail,num_dubb)
		else:
			UpdateStatus(-1,row['id'],num_succ,num_fail,num_dubb)
		return [status,msg]
			
	except Exception as e: 
		print (" Exception in IgnoreOrDeleteDoImportDigitMapping:  %s" % (str(e) ) )
		logger.error(" Exception in IgnoreOrDeleteDoImportDigitMapping:  %s" % (str(e) ) )
		DoError(row," Exception in IgnoreOrDeleteDoImportDigitMapping function:  %s" % (str(e)) )
		traceback.print_exc()
		UpdateStatus(-5,row['id'])
		return [-2,"Error in Importing"]

def  makeVendorDict():
	'''Makes dict with keys vendor names (resource.alias) , and values [resource_id,min_price,rate_table_id] '''
	#sql = """SELECT resource.alias,resource.resource_id,did_billing_plan.min_price,resource.rate_table_id from resource INNER JOIN did_billing_plan ON did_billing_plan.name = resource.alias where resource.ingress = 't' and resource.trunk_type2 = 1 """
	#sql  = """select rs.resource_id, rs.alias,rs.client_id,rs.rate_table_id  from resource rs LEFT JOIN client on rs.alias = client.name where rs.ingress=true and rs.trunk_type2=1 and client.client_type = 0"""
	sql  = """select rs.resource_id, rs.alias ,rs.client_id,rs.rate_table_id from resource rs LEFT JOIN client on rs.alias = client.name where rs.ingress=true and rs.trunk_type2=1 and client.client_type = 0 order by alias asc ;"""

	print(sql)
	try:
		pg_cur.execute(sql)
		result = pg_cur.fetchall()
		return_dict = {}
		for row in result:
			return_dict[row['alias']] = [row['resource_id'],row['client_id'],row['rate_table_id']]
	except Exception as e: 
		print (" Exception in makeVendorDict:  %s" % (str(e) ) )
		logger.error(" Exception in makeVendorDict:  %s" % (str(e) ) )
		traceback.print_exc()
		return {}
	return return_dict
	
	
def  makeClientDict():
	#'''Makes dict with keys vendor names (resource.alias) , and values [resource_id,min_price] '''
	#sql = """SELECT resource.alias,resource.resource_id,did_billing_plan.min_price ,resource.rate_table_id from resource INNER JOIN did_billing_plan ON did_billing_plan.id = resource.billing_rule where resource.egress = 't' and resource.trunk_type2 = 1 """
	
	sql  = """ select resource_id, alias,client_id  from resource where egress=true and trunk_type2=1 AND alias NOT LIKE '%_RElE_%' """
	print(sql)
	try:
		pg_cur.execute(sql)
		result = pg_cur.fetchall()
		return_dict = {}
		for row in result:
			#return_dict[row['alias']] = [row['resource_id'],row['min_price'],row['rate_table_id']]
			return_dict[row['alias']] = [row['resource_id'],row['client_id']]
	except Exception as e: 
		print (" Exception in makeVendorDict:  %s" % (str(e) ) )
		logger.error(" Exception in makeVendorDict:  %s" % (str(e) ) )
		traceback.print_exc()
		return {}
	return return_dict
	

def CreateVendor(vendor_name , route_strategy_id=0):
	print ("CreateVendor")
	media_type  = 2
	billing_rule_name = vendor_name + '_rule_name'
	mode    = 1
	sql = "SELECT currency_id FROM currency WHERE code = 'USA' LIMIT 1"
	pg_cur.execute(sql)
	currency_id = pg_cur.fetchone()
	if currency_id != None:
		currency_id = currency_id ['currency_id']
	else:
		sql =   "INSERT INTO currency (code) VALUES ('USA') returning currency_id "
		pg_cur.execute(sql)
		currency_id = pg_cur.fetchone()[0]
	print ("currency_id",currency_id)
		
	sql = """insert into client(name,mode,unlimited_credit,allowed_credit,currency_id,is_panelaccess,client_type,auto_invoicing,enough_balance) 
	values(%s,%s,%s,%s,%s,%s,%s,%s,%s) returning client_id"""
	pg_cur.execute(sql, (  vendor_name, mode,False, 0,currency_id, True,0, False, True )  )
	client_id = pg_cur.fetchone()[0]
	print ("New client id",client_id)
	
	sql ="insert into client_balance(client_id, balance) values (%s, 1)" % client_id 
	pg_cur.execute(sql)
	
	sql = "SELECT id FROM jurisdiction_country WHERE name = 'US'"
	pg_cur.execute(sql)
	jurisdiction_country = pg_cur.fetchone()
	if jurisdiction_country != None:
		jurisdiction_country_id  = jurisdiction_country['id']
	else:
		sql ="INSERT INTO jurisdiction_country(name) VALUES ('US') returning id"
		pg_cur.execute(sql)
		jurisdiction_country_id  = pg_cur.fetchone()[0]
	print ("jurisdiction_country_id ",jurisdiction_country_id )
	sql = """insert into rate_table(name, currency_id, jurisdiction_country_id, rate_type, jurisdiction_prefix,noprefix_min_length, noprefix_max_length, prefix_min_length, 
	prefix_max_length, jur_type,origination) values('%s',%s,%s,0,'1',10,10,11,11,2,true) returning rate_table_id """ % (vendor_name, currency_id, 
	jurisdiction_country_id)
	pg_cur.execute(sql)
	rate_table_id = pg_cur.fetchone()[0]
	#   create new bill rule
	sql  = "insert into did_billing_plan (did_price,channel_price,min_price,billed_channels,name) values ( 1, 1, 1, 1,'%s') returning id	" % billing_rule_name 
	pg_cur.execute(sql)
	new_bill_rule_id = pg_cur.fetchone()[0]
	#    create new egress
	sql = "insert into resource (alias,client_id,media_type,trunk_type2,ingress,billing_rule) values ('%s',%s,%s,%s,%s,%s) returning resource_id" % (vendor_name,
	client_id,media_type,  1,True, new_bill_rule_id)
	pg_cur.execute(sql)
	resource_id = pg_cur.fetchone()[0]
	
	sql = "insert into route_strategy (name) values ('%s') returning route_strategy_id" % (vendor_name)
	pg_cur.execute(sql)
	route_strategy_id= pg_cur.fetchone()[0]
	#insert into resource_prefix
	sql = "insert into resource_prefix (rate_table_id, route_strategy_id, resource_id) values (%s,%s,%s)" % (rate_table_id,route_strategy_id,resource_id)
	pg_cur.execute(sql)
	return 0
	
def CreateClient(client_name):
	print ("Create New Client")
	media_type  = 2
	billing_rule_name = client_name + '_rule_name'
	mode    = 1  #  means prepaid
	
	sql = "SELECT currency_id FROM currency WHERE code = 'USA' LIMIT 1"
	pg_cur.execute(sql)
	currency_id = pg_cur.fetchone()
	print ("currency_id ROW",currency_id)
	if currency_id != None:
		print ("currency_id != None")
		currency_id = currency_id ['currency_id']
	else:
		sql =   "INSERT INTO currency (code) VALUES ('USA') returning currency_id "
		
		pg_cur.execute(sql)
		currency_id = pg_cur.fetchone()[0]
	print ("currency_id no array!",currency_id)
		#currency_id = currency_id['currency_id']  
		   
	sql = """insert into client(name,mode,unlimited_credit,allowed_credit,currency_id,is_panelaccess,client_type,auto_invoicing,enough_balance) 
	values(%s,%s,%s,%s,%s,%s,%s,%s,%s) returning client_id"""
	pg_cur.execute(sql, (  client_name, mode,False, 0,currency_id, True,1, False, True )  )    
	client_id = pg_cur.fetchone()[0]
	print ("client_id",client_id)

	sql ="insert into client_balance(client_id, balance) values (%s, 1)" % client_id 
	pg_cur.execute(sql)
	
	sql = "SELECT id FROM jurisdiction_country WHERE name = 'US'"
	pg_cur.execute(sql)
	jurisdiction_country = pg_cur.fetchone()
	if jurisdiction_country != None:
		jurisdiction_country_id  = jurisdiction_country.get('id',None)
	else:
		 #    create us jurisdiction country
		sql ="INSERT INTO jurisdiction_country(name) VALUES ('US') returning id"
		pg_cur.execute(sql)
		jurisdiction_country_id = pg_cur.fetchone()[0]
		#jurisdiction_country_id = jurisdiction_country['id']
	#create rate table
	sql = """insert into rate_table (name, currency_id, jurisdiction_country_id, rate_type, jurisdiction_prefix,noprefix_min_length, noprefix_max_length, prefix_min_length, 
	prefix_max_length, jur_type,origination ) values ('%s',%s,%s,0,'1',10,10,11,11,2,true) returning rate_table_id """ % (client_name, currency_id, 
	jurisdiction_country_id)
	pg_cur.execute(sql)
	rate_table_id = pg_cur.fetchone()[0]
	#   create new bill rule
	sql  = "insert into did_billing_plan (did_price,channel_price,min_price,billed_channels,name,rate_table_id) values ( 1, 1, 1, 1,'%s',%s) returning id	" %( billing_rule_name,
	rate_table_id )
	pg_cur.execute(sql)
	new_bill_rule_id  = pg_cur.fetchone()[0]
	print ("new_bill_rule_id",new_bill_rule_id,"billing_rule_name ",billing_rule_name )

	#    create new egress
	sql = "insert into resource (alias,client_id,media_type,trunk_type2,egress,billing_rule,enough_balance) values ('%s',%s,%s,%s,%s,%s,'true') returning resource_id" % (client_name,
	client_id,media_type,  1,True, new_bill_rule_id)
	pg_cur.execute(sql)
	resource_id = pg_cur.fetchone()[0]
	print ("resource_id ",resource_id )
	return 0


def makeDidList():
	sql = """SELECT code from rate where did_type = 1 """
	pg_cur.execute(sql)
	result = pg_cur.fetchone()
	res_lst = []
	try:
		for row in result:
			res_lst.append(row[0])
	except:
		return res_lst
	return res_lst
	


def IgnoreOrDeleteDoImportDIDShort (scv_list,row,id_data,table_name,table_id_data,csv_reader_list) :
	print ("Entering IgnoreOrDeleteDoImportDIDSHORT ")
	status = 0 
	msg = "Done"
	action =  row.get('duplicate_type','delete')
	print ("action type is %s" % action)
	some_error= False
	num_succ  =0
	num_fail  =0
	num_dubb  =0
	foreign_id= row.get('foreign_id',0)
	client_id = foreign_id
	counter =1 
	
	try:
		for csv_row in csv_reader_list:  # loop over all rows 
			csv_row =stripSpace(csv_row)
			did= csv_row[0] if csv_row[0] != "" else None 
			Vendor_billing_plan = csv_row[1] if csv_row[1] != "" else None   
			client_id = csv_row[2] if csv_row[2] != "" else None
			
			if client_id == None:
				DoError(row,"Vendor_id can not be null",-4)
				num_fail = num_fail + 1
				counter = counter + 1 
				continue
			
			if Vendor_billing_plan == None:
				DoError(row,"Vendor_billing_plan can not be null",-4)
				num_fail = num_fail + 1
				counter = counter + 1 
				continue
			sql_v = """select id, name ,min_price ,rate_table_id from did_billing_plan where name = '%s' """ % Vendor_billing_plan
			try:
				pg_cur.execute(sql_v)
				res_v = pg_cur.fetchone()
				print ("res_v",res_v)
				if res_v != None:
					sell_billing_plan_id  = res_v['id']
					#sell_min_price  = res_v['min_price']
					#sell_rate_table_id  = res_v['rate_table_id']
				else:
					sell_billing_plan_id  = None
					#sell_min_price  = 0
					#buy_billing_plan_id  = None
					#buy_min_price  = 0
					#buy_name ="N/A"
				
			except Exception as e: 
				traceback.print_exc()
				num_fail = num_fail + 1
				counter = counter + 1 
				print (" Exception in IgnoreOrDeleteDoImportDIDSHORT:  %s" % (str(e) ) )
				logger.error(" Exception in IgnoreOrDeleteDoImportDIDSHORT  getting billing plans:  %s" % (str(e) ) )
				DoError(row," Exception in IgnoreOrDeleteDoImportDIDSHORT  getting billing plans:  %s" % (str(e) ) )					
					
			try: 
				did = int(did)
			except Exception as e: 
				traceback.print_exc()
				DoError(row,"Row number %s is incorrect %s"  % (counter, str(csv_row  ) ),  -4)
				num_fail = num_fail + 1
				counter = counter + 1 
				continue
			
			sql  = """SELECT resource_id FROM resource WHERE client_id = %s """ % client_id
			print (sql)
			pg_cur.execute (sql)
			result = pg_cur.fetchone()
			if result == None:
				num_fail = num_fail + 1
				counter = counter + 1 
				logger.error(" Non exisiting client_id in resource table  %s" % (client_id ) )
				DoError(row," Non exisiting client_id in resource table  %s" % (client_id ) )
			resource_id = result['resource_id']
			
			
			sql = """SELECT * FROM  did_billing_rel where did = '%s' """ % str(did)
			print (sql)
			pg_cur.execute(sql)
			result = pg_cur.fetchone()
			
			if result !=None:
				num_dubb +=1
				#num_succ -=1
				if action == 'delete':
					sql = """DELETE FROM  did_billing_rel where did = '%s' """ % str(did)
					print (sql)
					result = pg_cur.execute(sql)
					message = "Deleted row:  %s" % str(result)
					DoError(row,message,-1) 
					print(message)
					
				else:
					message = "Duplicate row:  %s" % str(csv_row)
					DoError(row,message,-1) 
					print(message)
					continue
					
					
			sql  = "INSERT INTO did_billing_rel ( did, sell_billing_plan_id, egress_res_id,start_date) VALUES ('%s',%s,%s,now()::date )  " %( did, sell_billing_plan_id,resource_id   )
			print (sql)
			pg_cur.execute(sql)
			num_succ +=1
			
		UpdateStatus(2,row['id'],num_succ,num_fail,num_dubb)

		return [status,msg]
			
	except Exception as e: 
		print (" Exception in IgnoreOrDeleteDoImportDIDShort:  %s" % (str(e) ) )
		logger.error(" Exception in IgnoreOrDeleteDoImportDIDShort:  %s" % (str(e) ) )
		DoError(row," Exception in IgnoreOrDeleteDoImportDIDShort function:  %s" % (str(e)) )
		traceback.print_exc()
		UpdateStatus(-5,row['id'])
		return [-2,"Error in Importing"]
				
				
				

def IgnoreOrDeleteDoImportDID (scv_list,row,id_data,table_name,table_id_data,csv_reader_list) :
	print ("Entering IgnoreOrDeleteDoImportDID ")
	status = 0 
	msg = "Done"
	action =  row.get('duplicate_type',None)
	print ("action type is %s" % action)
	some_error= False
	num_succ  =0
	num_fail  =0
	num_dubb  =0
	vendor_id = row.get('foreign_id',0)
	
	if action == None:
		action = 'delete'
	try:
		#did		vendor				client---Vendor_billing_plan,Client_billing_plan
		#121234		aaa					bbb -----,
		#0			1						2	, 
		counter= 1
		vendor_dict = makeVendorDict()  # dict with keys vendor aliases so we can check if the vendor does not exists
		client_dict = makeClientDict()  #  the same but for clients. Clients have egress= t in resource table. Vendors have ingress
		#did_list = makeDidList()
	
			
			
		for csv_row in csv_reader_list:  # loop over all rows 
			csv_row =stripSpace(csv_row)
			did= csv_row[0] if csv_row[0] != "" else None 
			vendor= csv_row[1] if csv_row[1] != ""  else None
			client = csv_row[2] if csv_row[2] != "" else None   
			Vendor_billing_plan = csv_row[3] if csv_row[3] != "" else None   
			Client_billing_plan = csv_row[4] if csv_row[4] != "" else None   
			
			if Vendor_billing_plan == None:
				DoError(row,"Vendor_billing_plan can not be null",-4)
				num_fail = num_fail + 1
				counter = counter + 1 
				continue
			if Client_billing_plan == None:
				DoError(row,"Client_billing_plan can not be null",-4)
				num_fail = num_fail + 1
				counter = counter + 1 
				continue
			
			sql_v = """select id, name ,min_price ,rate_table_id from did_billing_plan where name = '%s' """ % Vendor_billing_plan
			sql_c = """select id, name ,min_price ,rate_table_id from did_billing_plan where name = '%s' """ % Client_billing_plan
			print (sql_v,sql_c)
			try:
				pg_cur.execute(sql_v)
				res_v = pg_cur.fetchone()
				pg_cur.execute(sql_c)
				res_c = pg_cur.fetchone()
				print ("res_v",res_v,"res_c",res_c)
				
				if res_v != None:
					sell_billing_plan_id  = res_v['id']
					sell_min_price  = res_v['min_price']
					sell_rate_table_id  = res_v['rate_table_id']
				else:
					sell_billing_plan_id  = None
					sell_min_price  = 0
				if res_c != None:
					buy_billing_plan_id  = res_c['id']
					buy_min_price  = res_c['min_price']
					buy_rate_table_id  = res_c['rate_table_id']
					buy_name = res_c['name']
				else:
					buy_billing_plan_id  = None
					buy_min_price  = 0
					buy_name = "N/A"
					buy_rate_table_id = 0
					# lets add for the moment that client billing plan do not exists, then return error for the line (2018-09-09)
					DoError(row,"Client_billing_plan (%s) does not exists"% Client_billing_plan,-4)
					num_fail = num_fail + 1
					counter = counter + 1 
					continue
				
				
				
			except Exception as e: 
				traceback.print_exc()
				num_fail = num_fail + 1
				counter = counter + 1 
				print (" Exception in IgnoreOrDeleteDoImportDID:  %s" % (str(e) ) )
				logger.error(" Exception in IgnoreOrDeleteDoImportDID  getting billing plans:  %s" % (str(e) ) )
				DoError(row," Exception in IgnoreOrDeleteDoImportDID  getting billing plans:  %s" % (str(e) ) )
				
				
			
			try: 
				did = int(did)
			except Exception as e: 
				traceback.print_exc()
				DoError(row,"Row number %s is incorrect %s"  % (counter, str(csv_row  ) ),  -4)
				num_fail = num_fail + 1
				counter = counter + 1 
				continue
			if vendor == None:
				#traceback.print_exc()
				DoError(row,"vendor can not be null",-4)
				num_fail = num_fail + 1
				counter = counter + 1 
				continue
			else:
				try:
					vendor_id = vendor_dict[vendor][1]
					vendor_res_id = vendor_dict[vendor][0]
					vendor_rate_t_id = vendor_dict[vendor][2]
				except:
					DoError(row,"vendor do not exist: %s"%vendor,-4)
					num_fail = num_fail + 1
					counter = counter + 1 
					continue

			if client != None:
				if client  not in  client_dict :
					DoError(row,"Client name does not exist: %s  in row %s" %  (client,counter),-1)
					num_fail = num_fail + 1
					counter = counter + 1 
					#some_error= True
					continue
				else:
					client_id = client_dict[client][1]
					client_res_id = client_dict[client][0]
					clientName_new  =  client + "_RElE_"+ buy_name
					# create new client if not exists
					sql  = """SELECT resource_id FROM resource WHERE alias = '%s' """ % clientName_new
					print (sql)
					pg_cur.execute (sql)
					result = pg_cur.fetchone()
					if result == None:
						sql = """ INSERT INTO resource (alias, client_id, media_type, capacity, trunk_type2, egress, rate_table_id, enough_balance, billing_rule, route_strategy_id, t38)
                                SELECT '%s', client_id, media_type, capacity, trunk_type2, egress, %s, enough_balance, billing_rule, route_strategy_id,'true' FROM resource WHERE resource_id = %s
                                returning resource_id""" % ( clientName_new ,buy_rate_table_id ,client_res_id )
						print (sql)
						pg_cur.execute (sql)
						result = pg_cur.fetchone()
						client_new_res_id = result['resource_id']
					else:
						client_new_res_id = result['resource_id']
						print ("client_new_res_id",client_new_res_id)

			if vendor  not in  vendor_dict  or vendor_dict[vendor][1] == None:
				DoError(row,"Vendor name does not exist or has a minute price of null: %s  in row %s" %  (vendor,counter),-1)
				num_fail = num_fail + 1
				counter = counter + 1 
				continue
			
			
			sql = """INSERT INTO rate (rate_table_id, code, rate, min_time, interval, intra_rate, inter_rate,did_type,create_time,country,code_name,effective_date)
			SELECT resource.rate_table_id,'%s',did_billing_plan.min_price,6,6,did_billing_plan.min_price,
			did_billing_plan.min_price,1,current_timestamp(0),'','',current_timestamp(0)
			FROM resource 
			INNER JOIN did_billing_plan ON did_billing_plan.id = %s WHERE resource.resource_id = %s
			ORDER BY resource.rate_table_id DESC LIMIT 1
			returning rate_id;""" % ( did,sell_billing_plan_id , vendor_res_id ) #sell_rate_table_id 
			
			#print ("SQL FOR INSERT IN DID:  %s"  %  sql)
			#logger.debug("SQL FOR INSERT IN DID:  %s"  %  sql)
			try:
				#pg_cur.execute(sql)
				pass
			except Exception as e: 
				DoError(row,e)
				num_fail = num_fail + 1
				counter = counter + 1 
				continue
			
			'''
			sql = """INSERT INTO  rate (code,rate,rate_table_id, create_time,did_type  )  VALUES ( '%s', %s,%s, CURRENT_TIMESTAMP,2)""" % ( did,sell_min_price ,buy_rate_table_id )
			print ("SQL FOR INSERT IN DID:  %s"  %  sql)
			logger.debug("SQL FOR INSERT IN DID:  %s"  %  sql)
			pg_cur.execute(sql)
			'''
			
			sql  = "select product_id from product where name = 'ORIGINATION_STATIC_ROUTE'"
			pg_cur.execute(sql)
			try:
				product_id = pg_cur.fetchone()['product_id']
			except:
				product_id = 1
			
			# (resource.alias) , and values [resource_id,min_price,rate_table_id] 
			if action == 'ignore':  #or 'delete':
				
				sql = """SELECT count(*) as sum FROM product_items INNER JOIN product_items_resource ON product_items.item_id = product_items_resource.item_id
WHERE product_items_resource.resource_id = %s AND product_id = %s  AND strategy = 1 AND digits = '%s';""" % ( client_id ,product_id , did )
				print ( "SQL FOR IGNORE DID option: %s"  %  sql )
				logger.debug( "SQL FOR IGNORE DID option: %s"  %  sql )
				pg_cur.execute(sql)
				count= pg_cur.fetchone()[0]
				print ("count: ",count)
				if count >0 :
					some_error= True
					#UpdateStatus(-1,row['id'])
					#print ("num_dubb: ",num_dubb)
					message = "Duplicate row:  %s" % str(csv_row)
					DoError(row,message,-1) 
					print(message)
					num_dubb = num_dubb +1
					continue
				counter = counter + 1
				  
				#insert into product_items (product_id, digits, strategy, update_by) values ({$product_id} , '{$number}' , 1, '{$_SESSION['sst_user_name']}') returning item_i
				if client != None:
					sql = """insert into product_items (product_id, digits, strategy, update_by) values (%s , '%s' , 1, 'Import script') returning item_id""" % (product_id,did)
					print (sql)
					logger.debug(sql)
					pg_cur.execute(sql)
					try:
						item_id  = pg_cur.fetchone().get ("item_id")
					except Exception as e:
						print (e)
						item_id  = None
					if item_id  == None:
						message = "item_id  for client %s is missing" % client
						DoError(row,message,-1) 
						num_dubb = num_fail +1
						continue
					#Then we insert the DID to the product_item table:
					sql = """Insert into product_items_resource( item_id, resource_id ) values ( %s,%s)""" %  (item_id ,client_new_res_id  )
					print (sql)
					logger.debug(sql)
					pg_cur.execute(sql)
					
					sql = """INSERT INTO rate (rate_table_id, code, rate, min_time, interval, intra_rate, inter_rate,did_type,create_time,country,code_name,effective_date)
			SELECT resource.rate_table_id,'%s',did_billing_plan.min_price,6,6,did_billing_plan.min_price,
			did_billing_plan.min_price,2,current_timestamp(0),'','',current_timestamp(0)
			FROM resource LEFT JOIN did_billing_plan ON did_billing_plan.id = resource.billing_rule WHERE resource.resource_id = %s
			returning rate_id;""" % ( did, client_new_res_id ) 
					print (sql)
					pg_cur.execute(sql)
					#sell_billing_plan_id  = res_v['id']
					#sell_min_price  = res_v['min_price']
					#sell_rate_table_id  = res_v['rate_table_id']
					sql = """DELETE FROM resource_prefix WHERE rate_table_id = %s AND resource_id = %s""" %(sell_rate_table_id, vendor_res_id )  
					print (sql)
					try:
						pg_cur.execute(sql)
					except Exception as e:
						print(e)
					sql = """insert into resource_prefix (code, rate_table_id, resource_id, route_strategy_id) values ('%s', %s, %s, (SELECT route_strategy_id FROM route_strategy where name = 'ORIGINATION_ROUTING_PLAN'))""" % (did,sell_rate_table_id, vendor_res_id)  
					print (sql)
					try:
						pg_cur.execute(sql)
					except Exception as e:
						print(e)
					sql = """INSERT INTO did_billing_rel (did, did_billing_id, buy_billing_plan_id, sell_billing_plan_id) VALUES ('%s',  %s ,%s, %s )""" %(did,buy_billing_plan_id, buy_billing_plan_id, sell_billing_plan_id)
					print (sql)
					try:
						pg_cur.execute(sql)
					except Exception as e:
						print(e)
					num_succ = num_succ +1
			
			else:
				sql = """SELECT count(*) as sum FROM product_items INNER JOIN product_items_resource ON product_items.item_id = product_items_resource.item_id
WHERE product_items_resource.resource_id = %s AND product_id = %s  AND strategy = 1 AND digits = '%s';""" % ( client_id ,product_id , did )
				print ( "SQL FOR DELETE DID option: %s"  %  sql )
				logger.debug( "SQL FOR IGNORE DID option: %s"  %  sql )
				pg_cur.execute(sql)
				count= pg_cur.fetchone()[0]
				print ("count: ",count)
				if count >0 :
					sql = """DELETE FROM product_items where product_id = %s and digits = '%s'""" % (product_id, did)
					print(sql)
					pg_cur.execute() 
					counter = counter + 1
					if pg_cur.rowcount > 0 :
						num_dubb = num_dubb +1
						num_succ = num_succ -1
				
				  
				#Then we need to find the routing plan for that client:
				if client != None:
					sql = """select resource_id , route_strategy_id from resource where client_id = %s;""" %client_dict[client][1]  #"SELECT route_strategy_id from resource_prefix where resource_id  = '%s'" % client_dict[client][0]
					print (sql)
					logger.debug(sql)
					pg_cur.execute(sql)
					try:
						route_strategy_id = pg_cur.fetchone().get('route_strategy_id')
					except:
						message = "route_strategy_id for client %s is missing" % client
						DoError(row,message,-1) 
						num_dubb = num_dubb +1
						continue
					if route_strategy_id == None:
						message = "route_strategy_id for client %s is missing" % client
						DoError(row,message,-1) 
						num_dubb = num_dubb +1
						continue
					#getting product_id
					sql ="""select * from route where route_type = 2 AND route_strategy_id = %s""" % route_strategy_id
					print (sql)
					logger.debug(sql)
					pg_cur.execute(sql)
					static_route_id = pg_cur.fetchone().get('static_route_id')
					if static_route_id == None:
						message = "static_route_id for client %s is missing" % client
						DoError(row,message,-1) 
						num_dubb = num_dubb +1
						continue

					
					
					
					sql = """INSERT INTO  rate (code,rate,rate_table_id, create_time,did_type  )  VALUES ( '%s', %s,%s, CURRENT_TIMESTAMP,1) returning rate_id""" % ( did,sell_min_price , vendor_dict[vendor][2] ) #sell_rate_table_id 
					
					print ("SQL FOR INSERT IN DID:  %s"  %  sql)
					logger.debug("SQL FOR INSERT IN DID:  %s"  %  sql)
					pg_cur.execute(sql)
					rate_id = pg_cur.fetchone()['rate_id']
					sql = """INSERT INTO  rate (code,rate,rate_table_id, create_time,did_type  )  VALUES ( '%s', %s,%s, CURRENT_TIMESTAMP,2)""" % ( did,sell_min_price ,buy_rate_table_id )
					print ("SQL FOR INSERT IN DID:  %s"  %  sql)
					logger.debug("SQL FOR INSERT IN DID:  %s"  %  sql)
					pg_cur.execute(sql)
					
					if buy_billing_plan_id  != None and sell_billing_plan_id  != None:
						#assign time 
						sql = """INSERT INTO did_billing_rel (did, sell_billing_plan_id,buy_billing_plan_id,did_billing_id,rate_id,ingress_res_id, egress_res_id ,start_date  )
						 VALUES ('%s',%s,%s,%s,%s,%s,%s,now()::date) """ % (did, sell_billing_plan_id  ,buy_billing_plan_id,buy_billing_plan_id,rate_id,client_new_res_id,vendor_res_id )
						print(sql)
						#print ("sell_billing_plan_id",sell_billing_plan_id,"buy_billing_plan_id",buy_billing_plan_id )
						pg_cur.execute(sql)
						
						#Then we insert the DID to the product_item table:
						sql = """Insert into product_items ( product_id, digits,update_at) values ( %s, '%s',now()) returning item_id""" %  (static_route_id ,did  )
						print (sql)
						logger.debug(sql)
						try:
							pg_cur.execute(sql)
							item_id  = pg_cur.fetchone().get ("item_id")
						except: #duplication error
							sql = """select item_id from product_items where product_id = %s and digits = '%s'""" %  (static_route_id ,did  )
							print (sql)
							pg_cur.execute(sql)
							item_id = pg_cur.fetchone()['item_id']
							sql = """delete from product_items where product_id = %s and digits = '%s'""" %  (static_route_id ,did  )
							print (sql)
							pg_cur.execute(sql)
							sql = """delete from product_items_resource where item_id = %s """ %  item_id
							print (sql)
							pg_cur.execute(sql)
							sql = """Insert into product_items ( product_id, digits,update_at) values ( %s, '%s',now()) returning item_id""" %  (static_route_id ,did  )
							print (sql)
							pg_cur.execute(sql)
							item_id  = pg_cur.fetchone().get ("item_id")
						if item_id  == None:
							message = "item_id  for client %s is missing" % client
							DoError(row,message,-1) 
							num_dubb = num_dubb +1
							continue
						#Then we insert the DID to the product_item table:
						sql = """Insert into product_items_resource( item_id, resource_id ) values ( %s,%s)""" %  (item_id ,client_new_res_id  )
						print (sql)
						logger.debug(sql)
						pg_cur.execute(sql)
						
						
					num_succ = num_succ +1
				

		UpdateStatus(2,row['id'],num_succ,num_fail,num_dubb)

		return [status,msg]
			
	except Exception as e: 
		print (" Exception in IgnoreOrDeleteDoImportDID:  %s" % (str(e) ) )
		logger.error(" Exception in IgnoreOrDeleteDoImportDID:  %s" % (str(e) ) )
		DoError(row," Exception in IgnoreOrDeleteDoImportDID function:  %s" % (str(e)) )
		traceback.print_exc()
		UpdateStatus(-5,row['id'])
		return [-2,"Error in Importing"]



def IgnoreOrDeleteDoImportResourceBlock (scv_list,row,id_data,table_name,table_id_data,csv_reader_list) :
	print ("Entering IgnoreOrDeleteDoImportResourceBlock ")
	status = 0 
	msg = "Done"
	action =  row.get('duplicate_type',None)
	#print ("action type is %s" % action)
	some_error= False
	num_succ  =0
	num_fail  =0
	num_dubb  =0
	
	# Yes/No 
	
	#	SELECT "Resource"."resource_id" AS "Resource__resource_id", "Resource"."alias" AS "Resource__alias" FROM "resource" AS "Resource" WHERE ingress is true ORDER BY "alias" ASC
	#select alias from Resource WHERE ingress is true ORDER BY "alias" ASC ;
	#select name, company,client_id  from client  ORDER BY "name" ASC ;
	foreign_id = row.get('foreign_id',None)

	#print (" csv_reader_list", csv_reader_list,"scv_list",scv_list)
	if action == None:
		action = 'delete'
	
	
	try:

		
		if action == 'ignore':  # loop over every row in csv to decide if the row to be included in insert or not
			counter= 1
			for csv_row in csv_reader_list:  # loop over all rows 
				csv_row =stripSpace(csv_row)
				# drop header
				if csv_row[0].lower() =='ingress carrier':
					continue
				
				if  csv_row[7] != "" and  (int(csv_row[7])  < 0 or int(csv_row[7]) > 32  ):
					
					status = -1
					limits = limits +  " ANI Min is too low." 
					logger.debug( "Data is too big: row %s"  %  counter )
					message = "Error in line %s. %s " % (counter,limits )
					msg = "Done with errors.Please check log error file"
					DoError(row,message,-1) 
					some_error= True
					num_fail = num_fail +1 
					continue

				if csv_row[8] != "" and int(csv_row[8])  > 32 :
					limits = limits +   " ANI Max is too high."
					logger.debug( "Data is too big: row %s"  %  counter )
					message = "Error in line %s. %s " % (counter,limits )
					msg = "Done with errors.Please check log error file"
					DoError(row,message,-1) 
					some_error= True
					num_fail = num_fail +1 
					continue

				if  csv_row[9] != "" and  (int(csv_row[9])  < 0 or int(csv_row[9]) > 32 ) :
					limits = limits +  " DNIS Min is too low."
					logger.debug( "Data is too big: row %s"  %  counter )
					message = "Error in line %s. %s " % (counter,limits )
					msg = "Done with errors.Please check log error file"
					DoError(row,message,-1) 
					some_error= True
					num_fail = num_fail +1 
					continue

				if  csv_row[10] != "" and  int(csv_row[10]) > 32 :
					limits = limits +   " DNIS Max is too high."
					print ( "Data is too big: row %s"  %  counter )
					logger.debug( "Data is too big: row %s"  %  counter )
					message = "Error in line %s. %s " % (counter,limits )
					msg = "Done with errors.Please check log error file"
					DoError(row,message,-1) 
					some_error= True
					num_fail = num_fail +1 
					#UpdateStatus(-1,row['id'])
					continue
				
				
				
				
				if len(csv_row[3] )== 0:
					engress_res_id = None
					egress_group_id= None
				else:
					if  csv_row[14].lower() != "block by trunk group":
						sql = "select resource_id as id from Resource WHERE alias = '%s'"  % csv_row[3]
						pg_cur.execute(sql)
						try:
							engress_res_id= pg_cur.fetchone()['id']
						except:
							message = "engress_res_alias is not in row:  %s" % str(csv_row)
							DoError(row,message,-1) 
							num_fail = num_fail +1
							continue
					else:
						sql = "select group_id as id from trunk_group WHERE group_name = '%s'"  % csv_row[3]
						pg_cur.execute(sql)
						try:
							egress_group_id= pg_cur.fetchone()['id']
						except:
							message = "engress_group_alias is not in row:  %s" % str(csv_row)
							DoError(row,message,-1) 
							num_fail = num_fail +1
							continue
							
			
				if len(csv_row[1]) == 0:
					ingress_res_id = None
					ingress_group_id= None
				else:
					if  csv_row[14].lower() != "block by trunk group":
						sql = "select resource_id as id from Resource WHERE alias = '%s'"  % csv_row[1]
						pg_cur.execute(sql)
						try:
							ingress_res_id= pg_cur.fetchone()['id']
						except:
							message = "ingress_res_alias is not in row:  %s" % str(csv_row)
							DoError(row,message,-1) 
							num_fail = num_fail +1
							continue
							
							
					else:
						sql = "select group_id as id from trunk_group WHERE group_name = '%s'"  % csv_row[1]
						pg_cur.execute(sql)
						try:
							ingress_group_id= pg_cur.fetchone()['id']
						except:
							message = "ingress_group_alias is not in row:  %s" % str(csv_row)
							DoError(row,message,-1) 
							num_fail = num_fail +1
							continue
				
				if len(csv_row[2]) == 0:
					egress_client_id = None
				else:
					sql = "select client_id as id  from client where name = '%s'"  % csv_row[2]
					pg_cur.execute(sql)
					egress_client_id= pg_cur.fetchone()['id']
				
				if len(csv_row[0]) == 0:
					ingress_client_id = None
				else:
					sql = "select client_id as id  from client where name = '%s'"  % csv_row[0]
					pg_cur.execute(sql)
					try:
						ingress_client_id= pg_cur.fetchone()['id']
					except:
						ingress_client_id = None
				
				
				if csv_row[6] == None or len(csv_row[6]) == 0:
					time_profile_id =  None
				else:
					sql = "select time_profile_id as id  from time_profile where name = '%s'"  % csv_row[6]
					pg_cur.execute(sql)
					try:
						time_profile_id = pg_cur.fetchone()['id']
					except:
						time_profile_id = None
				
				
				if csv_row[11] == None or csv_row[11] == "" or csv_row[11].lower() != 'true' :
					ani_empty = False 
				else:
					ani_empty = True
				

				digit = csv_row[5] if csv_row[5] != "" else None 
				ani_prefix=    csv_row[4] if csv_row[4] != "" else None 
				ani_length= csv_row[7]
				ani_max_length= csv_row[8]
				dnis_length= csv_row[9]
				dnis_max_length= csv_row[10]
				
				if digit == None and ani_prefix == None and ingress_client_id == None and egress_client_id == None:
					# empty row
					num_fail = num_fail +1
					logger.debug( "empty row: row %s"  %  counter )
					message = "empty row %s "  % (counter )
					DoError(row,message,-1) 
					continue
				
				if csv_row[14].lower() == "block by trunk group":
					print ("block by trunk group")
					block_type=2
					where_code1 = ' ingress_res_id = %s ' %  ingress_res_id  if ingress_res_id  != None else " (ingress_res_id is null or ingress_res_id='' " 
					where_code2 = ' AND engress_res_id = %s ' %  engress_res_id  if engress_res_id != None else " AND  (engress_res_id is null or engress_res_id= ''" 
				else:
					block_type=1
					where_code1 = ' ingress_group_id = %s ' %  ingress_group_id  if ingress_group_id  != None else ' ingress_group_id is null' 
					where_code2 = ' AND egress_group_id = %s ' %  egress_group_id  if egress_group_id != None else ' AND  egress_group_id is null' 
				#ingress_res_id , engress_res_id,digit,ani_prefix , ani_length,dnis_length,ani_empty,ani_max_length ,dnis_max_length

				where_code3 = " AND digit = '%s'" %  digit if digit != None else "  AND (digit ='' or digit is null) " 
				where_code4 = " AND ani_prefix = '%s' " %  ani_prefix  if ani_prefix!= None else "  AND (ani_prefix = '' or ani_prefix is null) " 
				#where_code5 = ' AND ani_length = %s ' %  ani_length  if ani_length != "" else '  AND ani_length is null'   
				#where_code6 =' AND ani_max_length= %s ' %  ani_max_length if ani_max_length!= "" else ' AND ani_max_length is null'  
				#where_code7 = ' AND dnis_length = %s ' %  dnis_length  if dnis_length != "" else ' AND dnis_length is null' 
				#where_code8 = ' AND dnis_max_length= %s ' %  dnis_max_length if dnis_max_length!= "" else ' AND dnis_max_length is null'  
				#where_code9 = ' AND ingress_client_id = %s ' %  ingress_client_id  if ingress_client_id != None else ' AND ingress_client_id  is null' 
				#where_code10 = ' AND egress_client_id = %s ' %  egress_client_id if egress_client_id != None else ' AND  egress_client_id is null'
				#where_code11 = " AND type = '%s' "%block_type
				where_code = where_code1 +where_code2 +where_code3 +where_code4 #+where_code5 +where_code6 +where_code7 +where_code8 +where_code9+where_code10 #+where_code11
				

					
 
				
				sql = "select count(*) as count from resource_block where %s" % (where_code)
				print ( "SQL FOR IGNORE RESOURCE BLOCK option: %s"  %  sql )
				logger.debug( "SQL FOR IGNORE RESOURCE BLOCK option: %s"  %  sql )
				pg_cur.execute(sql)
				count= pg_cur.fetchone()[0]
				print ("count: ",count)
				if count >0 :
					some_error= True
					#UpdateStatus(-1,row['id'])
					#print ("num_dubb: ",num_dubb)
					message = "Duplicate row:  %s" % str(csv_row)
					DoError(row,message,-1) 
					num_dubb = num_dubb +1
					continue
				counter = counter + 1
				columns =  """ingress_res_id , engress_res_id , digit ,  time_profile_id , ingress_client_id , egress_client_id ,ani_prefix , ani_length , 
				dnis_length ,  ani_empty  , ani_max_length , dnis_max_length,ingress_group_id  ,egress_group_id """
				#values = "('%s',%s,'%s','%s' )" % (csv_row[0],foreign_id ,csv_row[1],csv_row[2] )
				sql = "INSERT INTO resource_block  ("+ columns+ ") VALUES  (%s,%s,%s,%s,%s,%s,%s,%s,%s,%s,%s,%s,%s,%s) "
				print ( "SQL FOR INSERT IN resource_block: " , sql, "ani_prefix",ani_prefix  )
				#0					1				2				3				4	5			6				7				8				9				10				11	
				#Ingress Carrier	Ingress Trunk	Egress Carrier	egress Trunk	ANI	DNIS	Time profile	ANI Length Min	ANI Length Max	DNIS Length Min	DNIS length Max	Block No ANI
				#Block No ANI	ANI	DNIS	Egress Carrier	egress Trunk	Ingress Carrier	Ingress Trunk	Time Profile	ANI Length Min	ANI Length Max	DNIS Length Min	DNIS Length Max
				#0				1	2			3			4				5					6			7					8				9				10				11
				pg_cur.execute(sql, (ingress_res_id , engress_res_id , digit ,  time_profile_id , ingress_client_id , egress_client_id ,ani_prefix,int(   csv_row[7]  ) if len( csv_row[7]  ) > 0 else None , 
				int(  csv_row[9]  ) if len(csv_row[9] ) > 0 else None ,  ani_empty ,  int(  csv_row[8] ) if len(csv_row[8]  ) > 0 else None ,  int(  csv_row[10] ) if len(csv_row[10] ) > 0 else None 
				,ingress_group_id,egress_group_id) )
				num_succ = num_succ +1
			UpdateStatus(2,row['id'],num_succ,num_fail,num_dubb)

			return [status,msg]
				
		else:
			counter= 1
			
			for csv_row in csv_reader_list:  # loop over all rows 
				csv_row =stripSpace(csv_row)
				if csv_row[0].lower() =='ingress carrier':
					continue
				imits = ""
				
				
				if len(csv_row ) == 0:
					msg = " Possible empty rows:  %s  on row: %s" % (str(csv_row), counter    )
					DoError(row,msg,0) 
					#some_error= True
					num_fail = num_fail +1 
					counter = counter + 1
					continue
					
				if  csv_row[7] != "" and  (int(csv_row[7])  < 0 or int(csv_row[7]) > 32  ):
					
					status = -1
					limits = limits +  " ANI Min is too low."
					logger.debug( "Data is too big: row %s"  %  counter )
					message = "Error in line %s. %s " % (counter,limits )
					msg = "Done with errors.Please check log error file"
					DoError(row,message,-1) 
					some_error= True
					num_fail = num_fail +1 
					continue

				if csv_row[8] != "" and int(csv_row[8])  > 32 :
					limits = limits +   " ANI Max is too high."
					logger.debug( "Data is too big: row %s"  %  counter )
					message = "Error in line %s. %s " % (counter,limits )
					msg = "Done with errors.Please check log error file"
					DoError(row,message,-1) 
					some_error= True
					num_fail = num_fail +1 
					continue

				if  csv_row[9] != "" and  (int(csv_row[9])  < 0 or int(csv_row[9]) > 32 ) :
					limits = limits +  " DNIS Min is too low."
					logger.debug( "Data is too big: row %s"  %  counter )
					message = "Error in line %s. %s " % (counter,limits )
					msg = "Done with errors.Please check log error file"
					DoError(row,message,-1) 
					some_error= True
					num_fail = num_fail +1 
					continue

				if  csv_row[10] != "" and  int(csv_row[10]) > 32 :
					limits = limits +   " DNIS Max is too high."
					print ( "Data is too big: row %s"  %  counter )
					logger.debug( "Data is too big: row %s"  %  counter )
					message = "Error in line %s. %s " % (counter,limits )
					msg = "Done with errors.Please check log error file"
					DoError(row,message,-1) 
					some_error= True
					num_fail = num_fail +1 
					#UpdateStatus(-1,row['id'])
					continue
				
				
				engress_res_id = None
				egress_group_id= None
				ingress_res_id = None
				ingress_group_id= None
				if len(csv_row[3] )== 0:
					engress_res_id = None
					egress_group_id= None
				else:
					if  csv_row[14].lower() != "block by trunk group":
						sql = "select resource_id as id from Resource WHERE alias = '%s'"  % csv_row[3]
						pg_cur.execute(sql)
						try:
							engress_res_id= pg_cur.fetchone()['id']
						except:
							message = "engress_res_alias is not in row:  %s" % str(csv_row)
							DoError(row,message,-1) 
							num_fail = num_fail +1
							continue
					else:
						sql = "select group_id as id from trunk_group WHERE group_name = '%s'"  % csv_row[3]
						pg_cur.execute(sql)
						try:
							egress_group_id= pg_cur.fetchone()['id']
						except:
							message = "engress_group_alias is not in row:  %s" % str(csv_row)
							DoError(row,message,-1) 
							num_fail = num_fail +1
							continue
							
			
				if len(csv_row[1]) == 0:
					ingress_res_id = None
					ingress_group_id= None
				else:
					if  csv_row[14].lower() != "block by trunk group":
						sql = "select resource_id as id from Resource WHERE alias = '%s'"  % csv_row[1]
						pg_cur.execute(sql)
						try:
							ingress_res_id= pg_cur.fetchone()['id']
						except:
							message = "ingress_res_alias is not in row:  %s" % str(csv_row)
							DoError(row,message,-1) 
							num_fail = num_fail +1
							continue
							
							
					else:
						sql = "select group_id as id from trunk_group WHERE group_name = '%s'"  % csv_row[1]
						pg_cur.execute(sql)
						try:
							ingress_group_id= pg_cur.fetchone()['id']
						except:
							message = "ingress_group_alias is not in row:  %s" % str(csv_row)
							DoError(row,message,-1) 
							num_fail = num_fail +1
							continue
				
				if len(csv_row[2]) == 0:
					egress_client_id = None
				else:
					sql = "select client_id as id  from client where name = '%s'"  % csv_row[2]
					pg_cur.execute(sql)
					egress_client_id= pg_cur.fetchone()['id']
				
				if len(csv_row[0]) == 0:
					ingress_client_id = None
				else:
					sql = "select client_id as id  from client where name = '%s'"  % csv_row[0]
					pg_cur.execute(sql)
					try:
						ingress_client_id= pg_cur.fetchone()['id']
					except:
						ingress_client_id = None
				
				if csv_row[6] == None or len(csv_row[6]) == 0:
					time_profile_id =  None
				else:
					sql = "select time_profile_id as id  from time_profile where name = '%s'"  % csv_row[6]
					pg_cur.execute(sql)
					try:
						time_profile_id = pg_cur.fetchone()['id']
					except:
						time_profile_id = None
				
				
				if csv_row[11] == None or csv_row[11] == "" or csv_row[11].lower() != 'true' :
					ani_empty = False 
				else:
					ani_empty = True
				#0					1				2				3				4	5			6				7				8				9				10				11	
				#Ingress Carrier	Ingress Trunk	Egress Carrier	egress Trunk	ANI	DNIS	Time profile	ANI Length Min	ANI Length Max	DNIS Length Min	DNIS length Max	Block No ANI
				#Block No ANI	ANI	DNIS	Egress Carrier	egress Trunk	Ingress Carrier	Ingress Trunk	Time Profile	ANI Length Min	ANI Length Max	DNIS Length Min	DNIS Length Max
				#0				1	2			3			4				5					6			7					8				9				10				11
				digit = csv_row[5] if csv_row[5] != "" else None 
				ani_prefix=    csv_row[4] if csv_row[4] != "" else None 
				ani_length= csv_row[7]
				ani_max_length= csv_row[8]
				dnis_length= csv_row[9]
				dnis_max_length= csv_row[10]
				
				
				if digit == None and ani_prefix == None and ingress_client_id == None and egress_client_id == None:
					# empty row
					num_fail = num_fail +1
					logger.debug( "empty row: row %s"  %  counter )
					message = "empty row %s "  % (counter )
					DoError(row,message,-1) 
					continue
					
				if csv_row[14].lower() == "block by trunk group":
					#print ("block by trunk group")
					block_type=2
					where_code1 = ' ingress_res_id = %s ' %  ingress_res_id  if ingress_res_id  != None else ' ingress_res_id is null' 
					where_code2 = ' AND engress_res_id = %s ' %  engress_res_id  if engress_res_id != None else ' AND  engress_res_id is null' 
				else:
					block_type=1
					where_code1 = ' ingress_group_id = %s ' %  ingress_group_id  if ingress_group_id  != None else ' ingress_group_id is null' 
					where_code2 = ' AND egress_group_id = %s ' %  egress_group_id  if egress_group_id != None else ' AND  egress_group_id is null' 
				#ingress_res_id , engress_res_id,digit,ani_prefix , ani_length,dnis_length,ani_empty,ani_max_length ,dnis_max_length

				where_code3 = " AND digit = '%s'" %  digit if digit != None else "  AND (digit ='' or digit is null) " 
				where_code4 = " AND ani_prefix = '%s' " %  ani_prefix  if ani_prefix!= None else "  AND (ani_prefix = '' or ani_prefix is null) " 
				#where_code5 = ' AND ani_length = %s ' %  ani_length  if ani_length != "" else '  AND ani_length is null'   
				#where_code6 =' AND ani_max_length= %s ' %  ani_max_length if ani_max_length!= "" else ' AND ani_max_length is null'  
				#where_code7 = ' AND dnis_length = %s ' %  dnis_length  if dnis_length != "" else ' AND dnis_length is null' 
				#where_code8 = ' AND dnis_max_length= %s ' %  dnis_max_length if dnis_max_length!= "" else ' AND dnis_max_length is null'  
				#where_code9 = ' AND ingress_client_id = %s ' %  ingress_client_id  if ingress_client_id != None else ' AND ingress_client_id  is null' 
				#where_code10 = ' AND egress_client_id = %s ' %  egress_client_id if egress_client_id != None else ' AND  egress_client_id is null'
				#where_code11   = " AND type = '%s' "%block_type
				where_code = where_code1 +where_code2 +where_code3 +where_code4 #+where_code5 +where_code6 +where_code7 +where_code8 +where_code9+where_code10#+where_code11
	
				sql = "DELETE FROM resource_block where %s" % (where_code)
				print ( "SQL FOR DELETE RESOURCE BLOCK option: %s"  %  sql )
				logger.debug( "SQL FOR DELETE RESOURCE BLOCK option: %s"  %  sql )
				pg_cur.execute(sql)
				if pg_cur.rowcount > 0 :
					num_dubb = num_dubb +1
					num_succ = num_succ -1
				else:
					pass
				counter = counter + 1
				columns =  """ingress_res_id , engress_res_id , digit ,  time_profile_id , ingress_client_id , egress_client_id ,ani_prefix , ani_length , 
				dnis_length ,  ani_empty  , ani_max_length , dnis_max_length,ingress_group_id,egress_group_id  """
				#values = "('%s',%s,'%s','%s' )" % (csv_row[0],foreign_id ,csv_row[1],csv_row[2] )
				sql = "INSERT INTO resource_block  ("+ columns+ ") VALUES  (%s,%s,%s,%s,%s,%s,%s,%s,%s,%s,%s,%s,%s,%s) "
				#print ( "SQL FOR INSERT IN resource_block: " ,sql,  ani_prefix ,digit  )
				#logger.debug( "SQL FOR INSERT IN CODE DECK: %s"  %  sql )
				pg_cur.execute(sql, (ingress_res_id , engress_res_id , digit ,  time_profile_id , ingress_client_id , egress_client_id ,ani_prefix,int(   csv_row[7]  ) if len( csv_row[7]  ) > 0 else None , 
				int(  csv_row[9]  ) if len(csv_row[9] ) > 0 else None ,  ani_empty ,  int(  csv_row[8] ) if len(csv_row[8]  ) > 0 else None , 
				 int(  csv_row[10] ) if len(csv_row[10] ) > 0 else None,ingress_group_id,egress_group_id  ) )
				num_succ = num_succ +1
			UpdateStatus(2,row['id'],num_succ,num_fail,num_dubb)
			return [status,msg]
			
		return [0,"Done"]
	except Exception as e: 
		print (" Exception in IgnoreOrDeleteDoImportResourceBlock:  %s" % (str(e) ) )
		traceback.print_exc()
		logger.error(" Exception in IgnoreOrDeleteDoImportResourceBlock:  %s" % (str(e) ) )
		DoError(row," Exception in IgnoreOrDeleteDoImportResourceBlock function:  %s" % (str(e)) )
		UpdateStatus(-5,row['id'])
		return [-2,"Error in Importing"]



def IgnoreOrDeleteDoImportAniDins(row,csv_reader_list):
	
	#{b'egress_id': b'null', b'egress_carrier_id': b'null', b'ingress_carrier_id': b'null', b'ingress_id': b'null', b'create_by': b'admin', b'carrier_type': b'1', b'number_filed': b'ani_prefix'}
	num_succ  =0
	num_fail  =0
	num_dubb  =0
	cnt = 0
	action =  row.get('duplicate_type','delete')
	br = 0
	
	
	options = loads( row['ext_attributes'].encode('utf-8') )
	egress_res_id  = None if options[b'egress_id'] == b'null' else int(options[b'egress_id'])
	egress_client_id = None if options[b'egress_carrier_id'] == b'null' else int(options[b'egress_carrier_id'])
	ingress_client_id = None if options[b'ingress_carrier_id'] == b'null' else int(options[b'ingress_carrier_id'])
	ingress_res_id  = None if options[b'ingress_id'] == b'null' else int(options[b'ingress_id'])
	number_filed = 0 if options[b'number_filed'] == b'ani_prefix' else 1
	carrier_type = int (options[b'carrier_type'] )
	print("External options",options)



	where_code1 = ' ingress_res_id = %s ' %  ingress_res_id  if ingress_res_id  != None else ' ingress_res_id is null' 
	where_code2 = ' AND engress_res_id = %s ' %  egress_res_id  if egress_res_id != None else ' AND  engress_res_id is null' 
	where_code3 = ' AND ingress_client_id = %s ' %  ingress_client_id  if ingress_client_id != None else ' AND ingress_client_id  is null' 
	where_code4 = ' AND egress_client_id = %s ' %  egress_client_id if egress_client_id != None else ' AND  egress_client_id is null'
	where_code_ids = where_code1 +where_code2 +where_code3 +where_code4




	egress_res_id
	if number_filed ==0:
		where_code = "ani_prefix"
	else:
		where_code = "digit"
	
	if number_filed ==0:
		sql_insert = """INSERT INTO resource_block (ingress_res_id, engress_res_id,ingress_client_id,egress_client_id  ,ani_prefix ,create_by    ) VALUES ( %s,%s,%s,%s,%s,%s )"""
	else:
		sql_insert = """INSERT INTO resource_block (ingress_res_id, engress_res_id  ,ingress_client_id,egress_client_id ,digit  , create_by   ) VALUES ( %s,%s,%s,%s,%s ,%s)"""





	try:
		for code in csv_reader_list:
			code =stripSpace(code)
			if br % 300 ==0:
				print ("Counter %s"%br)
			br +=1
			#print ("code",code)
			if code == []:
				num_fail +=1
				cnt +=1
				msg = " Empty rows: %s" % cnt
				DoError(row,msg,0) 
				continue
			
			matchObj = re.match(MATCH_ONLY_NUMBERS_AND_ALPHABET , code[0].replace(" ", ""), re.M|re.I)
			if not matchObj:
				msg = "ANI/DNIS is with wrong format: %s  in row number: %s" % ( code[0], cnt )
				DoError(row,msg,-1) 
				num_fail = num_fail +1 
				cnt = cnt + 1
				continue
			
			if number_filed ==0:
				ani_prefix = code[0]
				digit = None
				where_code = " ani_prefix= '%s'  and (digit = '' or digit is null)"  % ani_prefix
				
			else:
				ani_prefix = None
				digit = code[0]
				where_code = " (ani_prefix = '' or ani_prefix is null) and digit = '%s' "  % digit
			
			
			
			
			if action == 'ignore':
				sql = """select count(*) as count  from resource_block where  %s and %s""" % (where_code_ids ,where_code )
				#print (sql)
				pg_cur.execute (sql )
				count = pg_cur.fetchone()[0]
				#print (count)
				if count >0 :
					message = "Duplicate row number:  %s" % cnt
					#print (message)
					DoError(row,message,-1) 
					num_dubb = num_dubb +1
					cnt += 1
					continue
				num_succ += 1
				cnt +=1
				pg_cur.execute (sql_insert, (ingress_res_id, egress_res_id ,ingress_client_id,egress_client_id ,code[0] ,'[Manual Import]' )  )
				#UpdateStatus(2,row['id'],num_succ,num_fail,num_dubb)
				
			else:  #delete
				sql = """DELETE FROM resource_block where   %s and %s""" % (where_code_ids ,where_code )
				print ("SQL for deleting duplicate rows %s" % sql)
				pg_cur.execute (sql  )
				print ("Deleted rows %s",pg_cur.rowcount)
				if pg_cur.rowcount > 0 :
					num_dubb = num_dubb +1
					num_succ = num_succ -1
				else:
					pass
				
				num_succ += 1
				cnt +=1
				pg_cur.execute (sql_insert, (ingress_res_id, egress_res_id ,ingress_client_id,egress_client_id ,code[0] ,'[Manual Import]' )  )
		UpdateStatus(2,row['id'],num_succ,num_fail,num_dubb)
			
		
	except Exception as e: 
		traceback.print_exc()
		print (" Exception in ANIDNIS BLOCK:  %s" % (str(e) ) )
		logger.error(" Exception in ANIDNIS BLOCK :  %s" % (str(e) ) )
		DoError(row,"Exception in ANIDNIS BLOCK:  %s" % (str(e)) )
		UpdateStatus(-5,row['id'])
		return [-2,"Error in Importing"]
	
	
	
	
	#print (egress_id,ingress_carrier_id)
	return [-2,"testing"]


def IgnoreOrDeleteDoResourceTranslation(scv_list,row,id_data,table_name,table_id_data,csv_reader_list):
	
	#{b'egress_id': b'null', b'egress_carrier_id': b'null', b'ingress_carrier_id': b'null', b'ingress_id': b'null', b'create_by': b'admin', b'carrier_type': b'1', b'number_filed': b'ani_prefix'}
	num_succ  =0
	num_fail  =0
	num_dubb  =0
	cnt = 0
	action =  row.get('duplicate_type','delete')
	
	pg_cur.execute ( "select alias,ingress,resource_id from resource" )
	
	
	res_alias_dict = {el[0]:[el[1],el[2] ] for el in  list(pg_cur.fetchall()) }
	
	#print (res_alias_dict) 
	pg_cur.execute ( "select translation_name,translation_id  from digit_translation" )
	translation_name_dict = {el[0]:el[1] for el in  list(pg_cur.fetchall()) }


	#print ( translation_name_dict) 
	pg_cur.execute ( "select name,time_profile_id from time_profile"  )
	name_dict = {el[0]:el[1] for el in  list(pg_cur.fetchall()) }
	name_dict[''] = None
	#print ( name_dict ) 
	
	try:
		for code in csv_reader_list:
			code =stripSpace(code)
			print ("code",code)
		#["trunk_name","time_profile_name","target","code","action","chars","number_type","number_length",""]
			

			if code == []:
				num_fail +=1
				cnt +=1
				msg = " Empty rows: %s" % cnt
				DoError(row,msg,0) 
				continue
			if code == ["","",""]:
				num_fail +=1
				cnt +=1
				msg = " Empty rows: %s" % cnt
				DoError(row,msg,0) 
				continue
			
			if code[0] not in res_alias_dict:
				num_fail +=1
				cnt +=1
				msg = " trunk_name not exist in row: %s" % cnt
				DoError(row,msg,0) 
				continue
			

			if code[1] not in translation_name_dict :
				num_fail +=1
				cnt +=1
				msg = " translation_name %s not exist in row: %s" %(code[1], cnt )
				DoError(row,msg,0) 
				continue

			
			if code[2] not in name_dict :
				num_fail +=1
				cnt +=1
				msg = " time_profile_name %s not exist in row: %s" %(code[2], cnt )
				DoError(row,msg,0) 
				continue

			translation_id =  translation_name_dict [code[1]]
			resource_id = res_alias_dict[code[0]][1] 
			
			if code[2] != None:
				time_profile_id=  name_dict [code[2]]
			else:
				time_profile_id= None
			
			
			if action == 'ignore':
				sql = """select count(*) as count  from resource_translation_ref where  resource_id= %s""" #% ( )
				#print (sql)
				pg_cur.execute (sql,  resource_id )
				count = pg_cur.fetchone()
				print (count)
				if count >0 :
					message = "Duplicate row number:  %s" % cnt
					DoError(row,message,-1) 
					num_dubb = num_dubb +1
					cnt += 1
					continue
				num_succ += 1
				cnt +=1
				sql_insert = """insert into  resource_translation_ref  (resource_id , translation_id , time_profile_id   ) 
				VALUES ( %s,%s,%s ) """
				pg_cur.execute (sql_insert, (   resource_id , translation_id,time_profile_id   )  )
				#print (pg_cur.fetchone())
				#UpdateStatus(2,row['id'],num_succ,num_fail,num_dubb)
				
			else:  #delete
				#print ("resource_id", resource_id )
				sql = """DELETE from resource_translation_ref where  resource_id= %s""" %  resource_id 
				print (sql)
				pg_cur.execute (sql )
				print ("Deleted rows %s",pg_cur.rowcount)
				if pg_cur.rowcount > 0 :
					num_dubb = num_dubb +1
					num_succ = num_succ -1
				else:
					pass
				
				num_succ += 1
				cnt +=1
				sql_insert = """insert into  resource_translation_ref  (resource_id , translation_id , time_profile_id   ) 
				VALUES ( %s,%s,%s ) """
				pg_cur.execute (sql_insert, (   resource_id , translation_id,time_profile_id   )  )
				#print (pg_cur.fetchone())
		UpdateStatus(2,row['id'],num_succ,num_fail,num_dubb)
			
		
	except Exception as e: 
		traceback.print_exc()
		print (" Exception in ResourceDirection:  %s" % (str(e) ) )
		logger.error(" Exception in ResourceDirection :  %s" % (str(e) ) )
		DoError(row,"Exception in ResourceDirection:  %s" % (str(e)) )
		UpdateStatus(-5,row['id'])
		return [-2,"Error in Importing"]
	
	
	
	
	#print (egress_id,ingress_carrier_id)
	return [-2,"testing"]




def IgnoreOrDeleteDoResourceDirection(scv_list,row,id_data,table_name,table_id_data,csv_reader_list):
	
	#{b'egress_id': b'null', b'egress_carrier_id': b'null', b'ingress_carrier_id': b'null', b'ingress_id': b'null', b'create_by': b'admin', b'carrier_type': b'1', b'number_filed': b'ani_prefix'}
	num_succ  =0
	num_fail  =0
	num_dubb  =0
	cnt = 0
	action =  row.get('duplicate_type','delete')
	
	pg_cur.execute ( "select alias,ingress,resource_id from resource" )
	
	
	res_alias_dict = {el[0]:[el[1],el[2] ] for el in  list(pg_cur.fetchall()) }
	
	#print (res_alias_dict) 
	pg_cur.execute ( "select direction_id from resource_direction" )
	
	direction_id_lst = [el[0] for el in  list(pg_cur.fetchall()) ]
	#print ( direction_id_lst) 
	pg_cur.execute ( "select name,time_profile_id from time_profile"  )
	name_dict = {el[0]:el[1] for el in  list(pg_cur.fetchall()) }
	name_dict[''] = None
	print ( name_dict ) 
	
	action_map= {'add_prefix':1,'add_suffix':2, 'del_prefix':3, 'del_suffix':4  }
	number_type_map= {'all':0,'>':1, '=':2, '<':3  }
	try:
		for code in csv_reader_list:
			code =stripSpace(code)
			print ("code",code)
		#["trunk_name","time_profile_name","target","code","action","chars","number_type","number_length",""]
			

			if code == []:
				num_fail +=1
				cnt +=1
				msg = " Empty rows: %s" % cnt
				DoError(row,msg,0) 
				continue


			try: 
				action = action_map[code[4]]
			except:
				num_fail +=1
				cnt +=1
				msg = " Unsupported action type %s  in row: %s" % (code[4],cnt)
				DoError(row,msg,0) 
				continue
				
			try: 
				number_type = number_type_map[code[6]]
			except:
				num_fail +=1
				cnt +=1
				msg = " Unsupported number type %s  in row: %s" % (code[6],cnt)
				DoError(row,msg,0) 
				continue
			

			#["trunk_name","time_profile_name","target","code","action","chars","number_type","number_length",""]
			if code[0] not in res_alias_dict:
				num_fail +=1
				cnt +=1
				msg = " trunk_name not exist in row: %s" % cnt
				DoError(row,msg,0) 
				continue
			

			
			if code[1] not in name_dict :
				num_fail +=1
				cnt +=1
				msg = " time_profile_name %s not exist in row: %s" %(code[1], cnt )
				DoError(row,msg,0) 
				continue

				
			direction =1 if  res_alias_dict[code[0]][0] else 2
			resource_id = res_alias_dict[code[0]][1] 
			digits = code[3] 
			dnis = code[5]
			if code[1] != None:
				time_profile_id=  name_dict [code[1]]
			else:
				time_profile_id= None
			_type = 0  if code[2]  != 'ani' else 1
			if number_type != 0:
				number_length = code[7]
			else:
				number_length = None
				

				
			
			if action == 'ignore':
				sql = """select count(*) as count  from resource_direction where direction = %s and action= %s and dnis= %s and resource_id= %s and 
				time_profile_id= %s and type= %s and number_length= %s and number_type= %s""" #% ( )
				#print (sql)
				pg_cur.execute (sql, (direction, action, dnis, resource_id, time_profile_id, 
				_type, number_length, number_type) )
				count = pg_cur.fetchone()
				print (count)
				if count >0 :
					message = "Duplicate row number:  %s" % cnt
					DoError(row,message,-1) 
					num_dubb = num_dubb +1
					cnt += 1
					continue
				num_succ += 1
				cnt +=1
				sql_insert = """insert into  resource_direction  ( direction , action , digits ,  dnis , resource_id , time_profile_id , 
				type , number_length , number_type  ) VALUES ( %s,%s,%s,%s,%s,%s,%s,%s,%s  ) returning direction_id"""
				pg_cur.execute (sql_insert, (  direction , action , digits ,  dnis , resource_id , time_profile_id , 
				_type , number_length , number_type  )  )
				print (pg_cur.fetchone())
				#UpdateStatus(2,row['id'],num_succ,num_fail,num_dubb)
				
			else:  #delete
				sql = """DELETE FROM resource_direction where   direction = %s and action= %s and dnis= %s and resource_id= %s and 
				time_profile_id= %s and type= %s and number_length= %s and number_type= %s"""   
				print ("SQL for deleting duplicate rows %s" % sql)
				pg_cur.execute (sql,( direction, action, dnis, resource_id, time_profile_id, 
				_type, number_length, number_type)  )
				print ("Deleted rows %s",pg_cur.rowcount)
				if pg_cur.rowcount > 0 :
					num_dubb = num_dubb +1
					num_succ = num_succ -1
				else:
					pass
				
				num_succ += 1
				cnt +=1
				sql_insert = """insert into  resource_direction  ( direction , action , digits ,  dnis , resource_id , time_profile_id , 
				type , number_length , number_type  ) VALUES ( %s,%s,%s,%s,%s,%s,%s,%s,%s  ) returning direction_id"""
				pg_cur.execute (sql_insert, (  direction , action , digits ,  dnis , resource_id , time_profile_id , 
				_type , number_length , number_type  )  )
				
				print (pg_cur.fetchone())
		UpdateStatus(2,row['id'],num_succ,num_fail,num_dubb)
			
		
	except Exception as e: 
		traceback.print_exc()
		print (" Exception in ResourceDirection:  %s" % (str(e) ) )
		logger.error(" Exception in ResourceDirection :  %s" % (str(e) ) )
		DoError(row,"Exception in ResourceDirection:  %s" % (str(e)) )
		UpdateStatus(-5,row['id'])
		return [-2,"Error in Importing"]
	
	
	
	
	#print (egress_id,ingress_carrier_id)
	return [-2,"testing"]








def IgnoreOrDeleteDoRandomAniGeneration (scv_list,row,id_data,table_name,table_id_data,csv_reader_list) :
	print ("Entering IgnoreOrDeleteDoRandomAniGeneration ")
	status = 0 
	msg = "Done"
	action =  row.get('duplicate_type',None)
	print ("action type is %s" % action)
	some_error= False
	num_succ  =0
	num_fail  =0
	num_dubb  =0
	random_table_id = row.get('foreign_id',None)
	if random_table_id == None:
		UpdateStatus(-5,row['id'])
		return [-1,"foreign_id is missing. Abort!"]
	if action == None:
		action = 'delete'
	try:
		if action == 'ignore':  # loop over every row in csv to decide if the row to be included in insert or not
			counter= 1
			for csv_row in csv_reader_list:  # loop over all rows 
				csv_row =stripSpace(csv_row)
				if len(csv_row ) == 0:
					msg = " Possible empty rows:  %s  on row: %s" % (str(csv_row), counter    )
					DoError(row,msg,0) 
					#some_error= True
					num_fail = num_fail +1 
					counter = counter + 1
					continue
				
				if len(csv_row[0] ) > 32  :
					status = -1
					limits = ""
					limits = limits +  " random_ani  is bigger from 32 chars."
					print ( "Data is too big: row %s"  %  counter )
					logger.debug( "Data is too big: row %s"  %  counter )
					message = "Error in line %s. %s " % (counter,limits )
					msg = "Done with errors.Please check log error file"
					DoError(row,message,-1) 
					some_error= True
					num_fail = num_fail +1 
					#UpdateStatus(-1,row['id'])
					continue
				sql = "select count(*) as count from random_ani_generation where ani_number = '%s' and random_table_id = %s " % (csv_row[0],  random_table_id )
				print ( "SQL FOR IGNORE random_ani_generation DECK option: %s"  %  sql )
				logger.debug( "SQL FOR IGNORE random_ani_generation DECK option: %s"  %  sql )
				pg_cur.execute(sql)
				count= pg_cur.fetchone()[0]
				print ("count: ",count)
				if count >0 :
					some_error= True
					#UpdateStatus(-1,row['id'])
					#print ("num_dubb: ",num_dubb)
					message = "Duplicate row:  %s" % str(csv_row)
					DoError(row,message,-1) 
					num_dubb = num_dubb +1
					continue
				counter = counter + 1
				values = "('%s',%s )" % (csv_row[0],random_table_id )
				sql = "INSERT INTO random_ani_generation  ( ani_number, random_table_id   ) VALUES  %s " %  (values  )
				print ( "SQL FOR INSERT IN random_ani_generation  %s"  %  sql )
				logger.debug( "SQL FOR INSERT IN random_ani_generation : %s"  %  sql )
				pg_cur.execute(sql)
				num_succ = num_succ +1
			if some_error== False:
				UpdateStatus(2,row['id'],num_succ,num_fail,num_dubb)
			else:
				UpdateStatus(-1,row['id'],num_succ,num_fail,num_dubb)
			return [status,msg]
				
		else:
			counter= 1
			#print ("csv_reader_list: ",str(csv_reader_list))
			for csv_row in csv_reader_list:  # loop over all rows 
				csv_row =stripSpace(csv_row)
				if len(csv_row ) == 0:
					msg = " Possible empty rows:  %s  on row: %s" % (str(csv_row), counter    )
					DoError(row,msg,0) 
					#some_error= True
					num_fail = num_fail +1 
					counter = counter + 1
					continue
				
				
				if len(csv_row[0] ) > 32  :
					status = -1
					limits = "random_ani_generation  is bigger from 32 chars."
					print ( "Data is too big: row %s"  %  counter )
					logger.debug( "Data is too big: row %s"  %  counter )
					message = "Error in line %s. %s " % (counter,limits )
					msg = "Done with errors.Please check log error file"
					DoError(row,message,-1) 
					#UpdateStatus(-1,row['id'])
					some_error= True
					num_fail = num_fail +1
					continue
				sql = "DELETE FROM random_ani_generation where ani_number = '%s' and random_table_id= %s " % (csv_row[0],random_table_id )
				print ( "SQL FOR DELETE random_ani_generation option: %s"  %  sql )
				logger.debug( "SQL FOR DELETE random_ani_generation option: %s"  %  sql )
				pg_cur.execute(sql)
				if pg_cur.rowcount > 0 :
					num_dubb = num_dubb +1
					num_succ = num_succ -1
				else:
					pass
				counter = counter + 1
				values = "('%s',%s )" % (csv_row[0],random_table_id  )
				sql = "INSERT INTO random_ani_generation ( ani_number,  random_table_id ) VALUES  %s " %  (values  )
				print ( "SQL FOR INSERT IN random_ani_generation : %s"  %  sql )
				logger.debug( "SQL FOR INSERT IN random_ani_generation : %s"  %  sql )
				pg_cur.execute(sql)
				num_succ = num_succ +1
				print ("num_succ: "+ str(num_succ))
			if some_error== False:
				UpdateStatus(2,row['id'],num_succ,num_fail,num_dubb)
			else:
				UpdateStatus(-1,row['id'],num_succ,num_fail,num_dubb)
			return [status,msg]
			
		return [0,"Done"]
	except Exception as e: 
		print (" Exception in IgnoreOrDeleteDoRandom_ani_generation :  %s" % (str(e) ) )
		logger.error(" Exception in IgnoreOrDeleteDoRandom_ani_generation :  %s" % (str(e) ) )
		DoError(row," Exception in IgnoreOrDeleteDoRandom_ani_generation function:  %s" % (str(e)) )
		UpdateStatus(-5,row['id'])
		return [-2,"Error in Importing"]

def NoneTheColumns(scv_header_list,csv_reader_list,table_headers):
	'''Makes  entire column None if the label is missing from the header'''
	cnt = 0
	# New xsv formt will not have empty strings for column labels that are ignored, so we must add them 
	#print ("scv_header_list",scv_header_list,"csv_reader_list",csv_reader_list,"table_headers",table_headers)
	new_csv_reader_list = []
	#removing empty lines
	for line in csv_reader_list:
		if len(line) > 0 :
			 new_csv_reader_list.append (line)
	csv_reader_list =  list(new_csv_reader_list)
	
	size_of_row =  len(table_headers)-1
	size_of_column = len (csv_reader_list)
	null_row = [""] * size_of_row
	csv_list = []
	for _dummy in range (size_of_column):
		csv_list.append(  list(null_row) )     
	pos_dict = {}
	#print ("New csv list ",csv_reader_list )
	for header in scv_header_list:
		#print ("header:",header)
		if header == "":
			new_pos = None
		else:
			new_pos =table_headers.index (header.lower())
		pos_dict[cnt] = new_pos
		cnt +=1
	#print ("new csv_reader_list ",csv_reader_list,"pos_dict", pos_dict)
	big_cnt = 0
	try:
		for line in csv_reader_list:
			#print("line length", len(line))
			small_cnt = 0
			for elem in line:
				if pos_dict[small_cnt] == None:
					small_cnt += 1
					continue
				#print (big_cnt,len(csv_list),small_cnt,len(pos_dict))
				csv_list[big_cnt][pos_dict[small_cnt]] = elem 
				small_cnt += 1
				#print (csv_list)
			big_cnt += 1
	except Exception as e: 
		print ("Error in None the columns %s "%e)
		logger.error ("Error in None the columns %s "%e)
		DoError(0,"Headers mismach!")
		UpdateStatus(-4,row['id'])  # Status =-4 header mismatch!
		return 1
	csv_reader_list = csv_list
	#print ("Rearanged csv list",csv_reader_list)
	return csv_reader_list

def DoImport(csv_keys,scv_list,row,id_data,csv_reader_list):
	print (VERSION)
	table_name = {
	'Code' : 'code',
	'JurisdictionUpload':'jurisdiction_prefix',
	'ResourceBlock':'ResourceBlock'
	}
	table_return = {
	'Code' : 'code_id',
	'JurisdictionUpload':'id'
		}
	table_id_data = {
	'Code' : 'code_deck_id',
	'JurisdictionUpload':'prefix'
		}
	# check Headers for matching
	
	
	header_ok , csv_reader_list= CheckHeader( scv_list,row ,csv_reader_list)
	if not header_ok:
		print ("Headers do not match for %s   %s table !"% (str( scv_list[0] ) ,row['obj'] ) )
		logger.debug ("Headers do not match for %s   %s table !"% (str( scv_list[0] ) ,row['obj'] ) )
		DoError(row,"Headers  %s do not match for   %s table !"% (str( scv_list[0] ) ,row['obj']) )
		UpdateStatus(-4,row['id'])  # Status =-4 header mismatch!
		return [1,"Headers  %s do not match for   %s table !"% (str( scv_list[0] ) ,row['obj'])  ]
	
	
	if row['upload_type'] ==  8:   #['JurisdictionUpload']:
		number,msg = IgnoreOrDeleteDoImportJurPrefix (scv_list,row,id_data,table_name,table_id_data,csv_reader_list)
		return [number,msg]
	
	if row['upload_type'] ==  9:
		number,msg =IgnoreOrDeleteDoImportCodeDeck (scv_list,row,id_data,table_name,table_id_data,csv_reader_list)
		return [number,msg]
	

	if row['upload_type'] ==  22:
		number,msg =IgnoreOrDeleteDoRandomAniGeneration (scv_list,row,id_data,table_name,table_id_data,csv_reader_list)
			
	if row['upload_type'] ==  7:
		number,msg =IgnoreOrDeleteDoImportResourceBlock (scv_list,row,id_data,table_name,table_id_data,csv_reader_list)
		return [number,msg]
	
	if row['upload_type'] ==  12:
		number,msg =IgnoreOrDeleteDoImportRoutePlan (scv_list,row,id_data,table_name,table_id_data,csv_reader_list)
		return [number,msg]
	
	if row['upload_type'] ==  11:
		number,msg =IgnoreOrDeleteDoImportStaticRoutePlan (scv_list,row,id_data,table_name,table_id_data,csv_reader_list)
		return [number,msg]
	
	if row['upload_type'] ==  2:
		number,msg =IgnoreOrDeleteDoImportEgress (scv_list,row,id_data,table_name,table_id_data,csv_reader_list)
		return [number,msg]
	
	if row['upload_type'] ==  1:
		number,msg =IgnoreOrDeleteDoImportIngress (scv_list,row,id_data,table_name,table_id_data,csv_reader_list)
		return [number,msg]
	
	if row['upload_type'] ==  3:
		number,msg =IgnoreOrDeleteDoImportHost(scv_list,row,id_data,table_name,table_id_data,csv_reader_list)
		return [number,msg]
	
	if row['upload_type'] ==  6:
		number,msg =IgnoreOrDeleteDoImportDigitMapping(scv_list,row,id_data,table_name,table_id_data,csv_reader_list)
		return [number,msg]
	
	if row['upload_type'] ==  13:
		number,msg =IgnoreOrDeleteDoImportDID(scv_list,row,id_data,table_name,table_id_data,csv_reader_list)
		return [number,msg]
		
	if row['upload_type'] ==  130:
		number,msg =IgnoreOrDeleteDoImportDIDShort(scv_list,row,id_data,table_name,table_id_data,csv_reader_list)
		return [number,msg]
	
	if row['upload_type'] ==  21:
		number,msg =IgnoreOrDeleteDoImportAniDins(row,csv_reader_list)
		return [number,msg]
	
	if row['upload_type'] ==  4:
		number,msg =IgnoreOrDeleteDoResourceDirection(scv_list,row,id_data,table_name,table_id_data,csv_reader_list)
		return [number,msg]
	
	if row['upload_type'] ==  5:
		number,msg =IgnoreOrDeleteDoResourceTranslation (scv_list,row,id_data,table_name,table_id_data,csv_reader_list)
		return [number,msg]
	
	
	return [-1,"Unrecognized type!!!"]

def UpdateStatus(status_id,row_id,num_succ=0,num_fail=0,num_dubb=0 ):
	
	if status_id == -1:  # -1 is not recognized from UI
		status_id= -4
	if status_id==1:  # proccessing ,no finish time !
		sql = "update import_export_logs set status = %s ,success_numbers= %s, duplicate_numbers= %s,error_row = %s where id = %s" % (status_id,num_succ,num_dubb,num_fail,row_id  )
	else:
		sql = "update import_export_logs set status = %s ,finished_time = current_timestamp(0),success_numbers= %s, duplicate_numbers= %s,error_row = %s where id = %s" % (status_id,num_succ,
		num_dubb,num_fail,row_id)
	print ("SQL for Update Status: "+ sql )
	logger.debug ("SQL for Update Status: "+ sql )
	try:
		pg_cur.execute(sql)
	except Exception as e: 
		print (" Exception in UpdateStatusToWorking:  %s" % (str(e) ) )
		logger.error(" Exception in UpdateStatusToWorking  %s" % (str(e) ) )
		DoError(row_id," Exception in  UpdateStatusToWorking  :  %s" % (str(e) ) )
		return -1


def csv_unireader(f, encoding="utf-8"):
	for row in csv.reader(codecs.iterdecode(f, encoding)):
		yield [e.decode("utf-8") for e in row]

def doWork(file_path='',id_data= 43,server_logger = ''):
	global logger
	if server_logger != '':
		logger= server_logger
	number=0
	msg = "Success"
	if  file_path!='':
		if main() != 0:
			return [1,"Error in seting up script. Please check config file and DB"]
	
	if  file_path=='':
		sql = "SELECT * FROM import_export_logs WHERE status = 0 and log_type = 1 order by time desc limit 1"
	else:
		sql = "SELECT * FROM import_export_logs WHERE status = 0 and log_type = 1 and file_path='%s' order by time desc limit 1" % file_path
	logger.debug("SQL for taking upload queue: "+ sql )
	#print ("SQL for taking upload queue: "+ sql )
	try:
		pg_cur.execute(sql)
		result =pg_cur.fetchall() 
		print ("Result:", result )
		logger.debug("Result:"+ str(result) )
	except Exception as e: 
		print (" Exception in getting from import_export_logs:  %s" % (str(e) ) )
		logger.error(" Exception in getting import_export_logs  %s" % (str(e) ) )
		return [2,"Exception in getting import_export_logs  %s" % (str(e))  ]
	for row in result:
		print ("Object",row.get('obj',"N/A"))
		#logger.debug(row.get('obj',"N/A"))
		if row.get('obj',"N/A") in ["Code","JurisdictionUpload",'RandomAniGeneration','ResourceBlock','Route','Productitem','Egress','Ingress','ResourceIp',
		'DigitTranslation','DID','block list', 'ResourceDirection', 'ResourceTranslation','DID_SHORT']:
			
			#with open(row['file_path'], 'rb') as f:
				try:
					#num_lines = sum(1 for line in open(row['file_path']))
					num_lines2 = sum(1 for line in open(row['file_path'],newline='',mode = 'r'))
					print (num_lines2)
					f = open(row['file_path'],mode = 'r',newline='' )  #"Error in reading some rows from CSV file" newline=''
					#import chardet
					#_result = chardet.detect(f.readline())
					#print (_result)
				except IOError:
					UpdateStatus(-2,row['id']) # status = -2 , missing file !
					print("Exception in accessing cvs file" )
					logger.error("Exception in accessing cvs file" )
					return [2,"Exception in accessing cvs file"   ]
				except Exception as e: 
					print (" Exception in getting from import_export_logs:  %s" % (str(e) ) )
					logger.error(" Exception in getting import_export_logs  %s" % (str(e) ) )
					return [2,"Exception in getting import_export_logs  %s" % (str(e))  ]
					
				if False:
					pass
				else:
					repeat = True
					#with f as csvfile:
					UpdateStatus(1,row['id'])  # status =1 means 'in progress'
					encoding="utf-8"
					try:
						#import unicodecsv
						csv_reader =   csv.reader(f, delimiter=',', quotechar='|' )  #csv.reader(codecs.iterencode(codecs.iterdecode(csvfile, encoding), "utf-8"), delimiter=',', quotechar='|') 
						#print (csv_reader)
						csv_reader_list = [next(csv_reader)]
						csv_reader_list_copy = []
						line_num = 0
						error_count = 0 
						succ = 0
						while True:
							try:
								tmp_row =  next(csv_reader)
								csv_reader_list.append (tmp_row)
								csv_reader_list_copy.append (tmp_row)
								line_num +=1 
								succ +=1
							except csv.Error:
								line_num +=1
								error_count +=1
								UpdateStatus(-4,"Error in reading some rows from CSV file") # status = -2 , missing file !
								print ("Error in reading some rows from CSV file in line %s"%line_num )
								
							except StopIteration:
								print ("Iteration End")
								print ("Lines total: %s  error_count %s" % (line_num,error_count) )
								break
								
							except Exception as e:
								#UpdateStatus(-4,"Error in reading some rows from CSV file") # status = -2 , missing file !
								line_num +=1
								error_count +=1
								print ("Error in reading some rows from CSV file in line num %s"%line_num)
								logger.error("Error in reading some rows from CSV file in line num %s"%line_num)
						print ("Length of csv list", len(csv_reader_list),"error count", error_count,"succ",succ )
						if len(csv_reader_list) > 300:  # too big
							#csvfile.seek(0)
							for elem in csv_reader_list: 
								csv_keys = elem
								if len(elem) == 0:
									continue
								if  len( elem[0] ) != 0 : # NOT empty row header
									break
							
							csv_list=[ "("+','.join(map(str, csv_keys)) + ") "] 
							if row['obj'] == "JurisdictionUpload":
									csv_list[0]=csv_list[0].replace('country,','jurisdiction_country_name,').replace('state,','jurisdiction_name,')

							number,msg = DoImport(csv_keys,csv_list,row,id_data,csv_reader_list)
							return [number,msg]
						
						if row.get('upload_type',0) == 21:
							number,msg =IgnoreOrDeleteDoImportAniDins(row,csv_reader_list) # this do not have headers. Special case
						
							return number,msg 
						csv_keys = ""
						print ("entering for loop for csv list")
						for elem in list(csv_reader_list): 
							print ("elems_csv",elem)
							csv_keys = elem
							#csv_keys = next (csv_reader_list ) # this are the keys for dictionary
							#print ("csv_keys",csv_keys,type(csv_keys) ,len(csv_keys))
							if len(elem) == 0:
								continue
							if  len( elem[0] ) != 0 : # NOT empty row header
								break
							
						csv_list=[ "("+','.join(map(str, csv_keys)) + ") "] 
						if row['obj'] == "JurisdictionUpload":
								# (country,state,prefix,ocn,lata)  jurisdiction_country_name , jurisdiction_name
								csv_list[0]=csv_list[0].replace('country,','jurisdiction_country_name,').replace('state,',
								'jurisdiction_name,')
						row_string = ""
						counter = 0
						#print (len(csv_reader_list))
						for _row in csv_reader_list:
							#csv_reader_list_copy.append(_row)
							row_string = row_string+ "("+ ','.join( str(x) if isinstance( x, int )  else  "'"+str(x)+"'"  for x in _row) +  ") ,"    #"("+','.join(map(str, _row))
							#print (counter)
						
					except csv.Error as e:
						print ('file {}, line {}: {}'.format(filename, reader.line_num, e))
						DoError(row,'file {}, line {}: {}'.format(filename, reader.line_num, e) )
						DoError(row,message,status=-2)
						f.close()
						return [2,  'Error in reading CSV file {}, line {}: {}'.format(filename, reader.line_num, e)  ]
						#sys.exit('file {}, line {}: {}'.format(filename, reader.line_num, e))
						
					except Exception as e: 
						f.close()
						print ("Error in looping:"+  str(e) )
						traceback.print_exc()
						DoError(row,"Error in main loop:"+  str(e),-4 )
						return [2,"Undefined error in main program loop"  ]
						
					row_string  = row_string [:-1]
					csv_list. append(row_string)
					#print ("csv_list",csv_list)
					
					number,msg =DoImport(csv_keys,csv_list,row,id_data,csv_reader_list_copy)
					logger.debug ("Number: "+str(number) + "Message: "+msg)
					f.close()
	logger.debug ("Number to be returned: "+str(number) + " Message: "+msg)
	return [number,msg]



def main():
	global config ,pg_cur,logger
	config = configparser.ConfigParser()
	config.read('config.ini')

	if int(config["log"].get('by_time',0) )== 1:
		handler = TimedRotatingFileHandler(config["log"].get ("import_script_path","import_script.log"),when=config["log"].get ("import_script_when",'m'),
		interval= int( config["log"].get ("import_script_interval",0) ),backupCount= int( config["log"].get ("import_script_backupCount",5)  ) )
	else: # by size
		handler = RotatingFileHandler(config["log"].get ("import_script_path","import_script.log"),maxBytes=int( config["log"].get ("import_script_maxBytes", 1024*1024*128 ) ),
		backupCount= int( config["log"].get ("import_script_backupCount",5)  ) )
	
	formatstr = '%(asctime)s - %(name)s - %(levelname)s - %(message)s'
	formatstr = '%(asctime)s: %(levelname)s:  %(message)s'
	formatter = logging.Formatter(formatstr)
	handler.setFormatter(formatter)
	logger.addHandler(handler)
	
	if int (config["log"].get ("debug_level",0) ) == 1: 
		logger.setLevel(logging.DEBUG)
	else:
		logger.setLevel(logging.INFO)
	
	try:
		pg_conn, pg_cur = connect_to_postgresql(config["POSTGRES"]["hostaddr"],int(config["POSTGRES"]["port"]), config["POSTGRES"]["dbname"], config["POSTGRES"]["user"],config["POSTGRES"]["password"]) #config["POSTGRES"]["password"]
	except Exception as e: 
		print (" Exception in connect_to_postgresql:  %s" % (str(e) ) )
		logger.error(" Exception in connect_to_postgresql:  %s" % (str(e) ) )
		return -1
	return 0
	
	
	#| cmd | rate_table_id | status | end_date | log_id | is_ocn_lata | date_format | rates_file_cmd | rates_file | code_name_match 
	
	
	#curl -H "Content-Type: application/json" -X POST -d '{"cmd":"",rate_table_id":"","is_ocn_lata":"","date_format":""," rates_file_cmd ":"","rates_file ":""}' http://158.69.203.19:6060/api/v1/importfile
	
	
if __name__ == '__main__':
	res = main()
	
	if res==0:
		while True:
			
			time.sleep(1)
			doWork()
			break
			time.sleep(60)
			
			
			
